package org.ow2.choreos.chordevelopment.analysis.depanalyzer.analysis.intra;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.ow2.choreos.chordevelopment.analysis.depanalyzer.analysis.core.RippleEffectCalculator;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Choreography;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Participant;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.TaskStep;

import edu.uci.ics.jung.graph.DirectedGraph;

/**
 * Analyzes a single choreography (and its subchoreographies)
 * @author goliva@ime.usp.br
 *
 */
public class IntraChoreographyAnalyzer {

	private DirectedGraph<Participant, ParticipantLink> participantCallGraph;
	
	public IntraChoreographyAnalyzer(Choreography choreography){

		ParticipantCallGraphBuilder callGraphBuilder = 
				new ParticipantCallGraphBuilder();
		
		participantCallGraph = callGraphBuilder.build(choreography);
	}
	
	public Collection<Participant> whatIfRemove(Participant participant){
				
		RippleEffectCalculator<Participant, ParticipantLink> rippleCalc = 
				new RippleEffectCalculator<Participant, ParticipantLink>();
		
		Collection<Participant> impactedParticipants = 
				rippleCalc.calculate(participantCallGraph, participant);
		
		return impactedParticipants;
	}
	
	public Set<Operation> getOperationsFromParticipant(Participant participant){
		
		Set<Operation> operations = new HashSet<Operation>();
		
		Collection<ParticipantLink> incommingEdges = 
				participantCallGraph.getIncidentEdges(participant);
		
		for(ParticipantLink incommingEdge : incommingEdges){
			for(String calledOperation : incommingEdge.getCalledOperations()){
				Operation operation = 
						new Operation(calledOperation, participant);
				operations.add(operation);
			}
		}
		
		return operations;
	}
	
	/**
	public Collection<Participant> whatIfModifyOrRemove(Operation operation){
		
		Collection<ParticipantLink> incommingEdges = 
				participantCallGraph.getIncidentEdges(
						operation.getParticipant());
		
		for(ParticipantLink incommingEdge : incommingEdges){
			
		}
	}
	*/
	
	
}