package org.ow2.choreos.chordevelopment.analysis.depanalyzer.analysis.intra;

import java.util.LinkedList;
import java.util.Queue;

import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Choreography;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.Participant;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.SubchoreographyStep;
import org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity.TaskStep;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;

/**
 * Builds the Participant Call Graph for a Single Choreography
 * @author goliva@ime.usp.br
 *
 */
public class ParticipantCallGraphBuilder {

	public DirectedGraph<Participant, ParticipantLink> build(
			Choreography choreography) {

		DirectedGraph<Participant, ParticipantLink> participantCallGraph = 
				new DirectedSparseGraph<Participant, ParticipantLink>();
		
		Queue<Choreography> choreographyQueue =	new LinkedList<Choreography>();
		choreographyQueue.add(choreography);
		
		processChoreographies(choreographyQueue, participantCallGraph);
		
		return participantCallGraph;
	}
	
	public void processChoreographies(Queue<Choreography> choreographyQueue,
			DirectedGraph<Participant, ParticipantLink> participantCallGraph){
		
		while(!choreographyQueue.isEmpty()){
			Choreography choreography = choreographyQueue.poll();
			fillCallGraph(choreography, participantCallGraph);
		
			//Adds the subchors to the queue
			for(SubchoreographyStep subchorStep : 
				choreography.getSubchoreographySteps()){
				
				choreographyQueue.add(subchorStep.getSubChoreography());
			}
		}
	}
	
	public void fillCallGraph(Choreography choreography, 
			DirectedGraph<Participant, ParticipantLink> callGraph){
		
		//Extract calls among participants
		for(TaskStep taskStep : choreography.getTaskSteps()){
			Participant client = taskStep.getInitiatingParticipant();
			Participant provider = taskStep.getTargetParticipant();
			ParticipantLink link = callGraph.findEdge(client, provider);
			String calledOperation = createLabel(taskStep, provider);
			if(link == null){
				link = new ParticipantLink("", calledOperation);
				callGraph.addEdge(link, client, provider);
			}
			else{
				link.addCalledOperation(calledOperation);
			}
			
		}
	}

	private String createLabel(TaskStep taskStep, Participant provider) {
		
		String calledOperation = provider.getName() + "." + 
				taskStep.getName() + "()";
		
		return calledOperation;
	}
}