
package org.ow2.choreos.chordevelopment.analysis.depanalyzer.entity;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class ChoreographySchema {

	private Step startStep;
	private Set<FinalStep> finalSteps;
	private Set<GatewayStep> gatewaySteps;
	private Set<TaskStep> taskSteps;
	private Set<SubchoreographyStep> subchorSteps;
	private Set<CallChoreographyStep> callChorSteps;
	
	private Map<String, Step> allSteps;
	private Set<Transition> transitions;
	
	protected ChoreographySchema(){
		allSteps = new HashMap<String, Step>();
		
		finalSteps = new HashSet<FinalStep>();
		gatewaySteps = new HashSet<GatewayStep>();
		
		taskSteps = new HashSet<TaskStep>();		
		subchorSteps = new HashSet<SubchoreographyStep>();
		callChorSteps = new HashSet<CallChoreographyStep>();
		
		transitions = new HashSet<Transition>();
	}
	
	protected void addStep(Step step){
		step.plugIntoChoreography(this);
	}
	
	protected void setSteps(Collection<Step> steps){
		this.allSteps.clear();
		
		this.startStep = null;
		this.finalSteps.clear();
		this.subchorSteps.clear();
		this.callChorSteps.clear();
		
		for(Step step : steps){
			addStep(step);
		}
	}
	
	protected Step getStep(String ID){
		return allSteps.get(ID);
	}
	
	protected void setStartStep(StartStep startStep){
		this.allSteps.put(startStep.getID(), startStep);
		this.startStep = startStep;
	}
	
	protected void addFinalStep(FinalStep finalStep){
		this.allSteps.put(finalStep.getID(), finalStep);
		this.finalSteps.add(finalStep);
	}
	
	protected void addGatewayStep(GatewayStep gatewayStep){
		this.allSteps.put(gatewayStep.getID(), gatewayStep);
		this.gatewaySteps.add(gatewayStep);
	}
	
	protected void addTaskStep(TaskStep taskStep){
		this.allSteps.put(taskStep.getID(), taskStep);
		this.taskSteps.add(taskStep);
	}
	
	protected void addSubchorStep(SubchoreographyStep subchorStep){
		this.allSteps.put(subchorStep.getID(), subchorStep);
		this.subchorSteps.add(subchorStep);
	}
	
	protected void addCallChorStep(CallChoreographyStep callChorStep){
		this.allSteps.put(callChorStep.getID(), callChorStep);
		this.callChorSteps.add(callChorStep);
	}
	
	protected void addTransition(Transition transition){
		this.transitions.add(transition);
	}

	public void setTransitions(Collection<Transition> transitions) {
		for(Transition transition : transitions){
			addTransition(transition);
		}
	}

	public Set<TaskStep> getTaskSteps() {
		return taskSteps;
	}

	public Set<SubchoreographyStep> getSubchoreographySteps() {
		return subchorSteps;
	}
}