#include <fstream> 
#include "definitions.h"
#include "BPMNStack.h"


typedef struct rOrder {
	int root; // id of the root node
	int leaf; // id of the leaf node
	int type_root; // type of the root node 
}nodeOrder;

class OperationGraph
{
	
private:
	int size;
	int ordersize;
	BPMNUnitTask** Nodes;

	nodeOrder* Order;
	float graphSrt; // SRT after the reduction
	float graphEc; // EC after the reduction
	float graphAva;
	float graphRel;
	
	void sequence_reduction(int root);
	void fork_reduction(int root, int leaf);
	void xor_reduction(int root, int leaf);
	void or_reduction(int root, int leaf);
	void loop_reduction(); // will explore the graph and reduce all loop nodes;
	void computeOrder();
	void insert(nodeOrder v);
	void reduce(int root);
	int isReducible(int root);
	
	int isOpen(BPMNUnitTask *e);
	int isClosed(BPMNUnitTask *e);
	int isOperation(BPMNUnitTask *e);

	void orderOpen(BPMNUnitTask *u, BPMNStack *Sd, BPMNStack *St, BPMNStack *Se);
	void orderClosed(BPMNUnitTask *u, BPMNStack *Sd, BPMNStack *St, BPMNStack *Se);	
	void orderOperation(BPMNUnitTask *u, BPMNStack *Sd, BPMNStack *St, BPMNStack *Se);	

	int getRootNode(int root);

	float max(float a, float b);

public:

	OperationGraph(); 
	~OperationGraph();
	void loadGraph(char* modelfile, char *qosfile, char *probfile);
	void reloadGraph(char* modelfile, char *qosfile, char *probfile);
	void loadGraphSrt(char* modelfile, char *qosfile, char *probfile, float powerMin, float powerMax);
	void reloadGraphSrt(char* modelfile, char *qosfile, char *probfile, float powerMin, float powerMax);
	int getSize();
	void printGraphId();
	float getSrt();
	float getEc();
	float getAva();
	float getRel();	
/*	Read a graph config in graphfile and 
	load it in *Nodes
*/
	void reduce_with_order(); // Apply reduction with a precomputed order
	void reduce_lex_order(); // Use the lexicographic order
	void reduce_rand_order(); // Apply the reduction with a random shuffle order

};


