package it.univaq.di.choreos.coordinationprotocol.algorithm;

import it.univaq.di.choreos.coordinationprotocol.algorithm.impl.CoordinationDelegateAlgorithmImpl;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class CoordinationDelegateFactory {
	
	static final Logger logger = LoggerFactory.getLogger(CoordinationDelegateFactory.class);
	
	private static Map<CoordinationDelegateID, CoordinationDelegateAlgorithm> CDS = Collections.synchronizedMap(new HashMap<CoordinationDelegateID, CoordinationDelegateAlgorithm>());

	private static Map<String, CoordinationDelegateData> coordsData = Collections.synchronizedMap(new HashMap<String, CoordinationDelegateData>());


	private CoordinationDelegateFactory() {
	}
	
	
	public synchronized static CoordinationDelegateAlgorithm getCoordinationDelegateAlgorithm(CoordinationDelegateID id) {
		CoordinationDelegateAlgorithm result = CDS.get(id);
		if (result==null) {
			CoordinationDelegateData data = coordsData.get(id.getCoordinatedServiceEndpointName());
			State initialState = CoordmetamodelFactory.eINSTANCE.createState();
			initialState.setName(data.getInitialState().getName());
			GlobalStateManager globalStateManager = new GlobalStateManager(initialState);
			
			result = new CoordinationDelegateAlgorithmImpl(id, data.getCoordModel(), globalStateManager, data.getCoordinationChannelManager());
			CDS.put(id, result);
		}
		return result;
	}

	public synchronized static void addCoordinationDelegateData(String coordinatedServiceEndpointName, CoordinationDelegateData data) {
		coordsData.put(coordinatedServiceEndpointName, data);
	} 

}
