package it.univaq.di.choreos.coordinationprotocol.algorithm.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationChannelManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateAlgorithm;
import it.univaq.di.choreos.coordinationprotocol.algorithm.ForwardAndDiscardMessagesCallback;
import it.univaq.di.choreos.coordinationprotocol.algorithm.GlobalStateManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.AdditionalMessage;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.ContextData;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Tuple;

import java.util.Iterator;
import java.util.concurrent.atomic.AtomicBoolean;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CoordinationDelegateAlgorithmImpl implements CoordinationDelegateAlgorithm {

	final Logger logger = LoggerFactory.getLogger(CoordinationDelegateAlgorithmImpl.class);

	private CoordinationDelegateID id;

	// private CoordinationDelegateManager manager;
	private GlobalStateManager globalStateManager;
	private AtomicBoolean updateCurrentStateReceived;
	private CoordinationChannelManager coordinationChannelManager;
	private Coord coord;

	public CoordinationDelegateAlgorithmImpl(CoordinationDelegateID id, Coord coord, GlobalStateManager globalStateManager, CoordinationChannelManager coordinationChannelManager) {
		this.id = id;
		this.coord = coord;
		this.globalStateManager = globalStateManager;
		updateCurrentStateReceived = new AtomicBoolean(false);
		this.coordinationChannelManager = coordinationChannelManager;
	}

	public void handleRules(AllowedOperation request, Participant coordinationDelegateFrom, ForwardAndDiscardMessagesCallback messagesCallback, ContextData contextData) {

		logger.debug("begin handleRules");

		// BEGIN RULE 1
		logger.debug("begin rule1");
		if (!isTupleIntheAlphabet(request)) {
			messagesCallback.discardMessage(contextData);
			return;
		}

		if (isTupleNotAllowedFromCurrentState(request)) {
			waitUpdateState(request);
		}

		if (isTupleAllowedFromCurrentState(request)) {
			// t is allowed from s
			Tuple t = getTuple(request);
			messagesCallback.forwardMessage(contextData);
			updateGlobalState(t);
			sendUpdateCurrentState(t, coordinationDelegateFrom);
		}

		logger.debug("end handleRules");
	}
	
	public void receiveUpdateCurrentState(AdditionalMessage additionalMessage) {
		logger.debug("begin receiveUpdateCurrentState. additionalMessage=" + additionalMessage);
		State newState = CoordmetamodelFactory.eINSTANCE.createState();
		newState.setName(additionalMessage.getState());
		globalStateManager.updateGlobalState(newState);
		notifyUpdateState();
		logger.debug("end receiveUpdateCurrentState");
	}

	public CoordinationDelegateID getId() {
		return id;
	}

	public void setId(CoordinationDelegateID id) {
		this.id = id;
	}


	private boolean isTupleIntheAlphabet(AllowedOperation request) {
		for (Iterator<Tuple> iterator = coord.getTuples().iterator(); iterator.hasNext();) {
			Tuple tuple = iterator.next();
			if (tuple.getAllowedOperation().getName().equals(request.getName())) {
				return true;
			}
		}
		return false;
	}

	private boolean isTupleAllowedFromCurrentState(AllowedOperation request) {
		for (Iterator<Tuple> iterator = coord.getTuples().iterator(); iterator.hasNext();) {
			Tuple tuple = iterator.next();
			if (tuple.getAllowedOperation().getName().equals(request.getName())) {
				if (globalStateManager.isCurrentState(tuple.getSourceState())) {
					return true;
				}
			}
		}
		return false;
	}

	private boolean isTupleNotAllowedFromCurrentState(AllowedOperation request) {
		boolean result = false;
		for (Iterator<Tuple> iterator = coord.getTuples().iterator(); iterator.hasNext();) {
			Tuple tuple = iterator.next();
			if (tuple.getAllowedOperation().getName().equals(request.getName())) {
				if (!globalStateManager.isCurrentState(tuple.getSourceState())) {
					result = true;
				} else {
					result = false;
					break;
				}

			}
		}
		return result;
	}

	private void waitUpdateState(AllowedOperation request) {
		synchronized (updateCurrentStateReceived) {
			while (!updateCurrentStateReceived.get()) {
				try {
					updateCurrentStateReceived.wait();
				} catch (InterruptedException e) {
					logger.error(e.getMessage());
				}
			}
		}
	}
	
	private void notifyUpdateState() {
		synchronized (updateCurrentStateReceived) {
			updateCurrentStateReceived.set(true);
			updateCurrentStateReceived.notifyAll();
		}
	}

	private Tuple getTuple(AllowedOperation request) {
		for (Iterator<Tuple> iterator = coord.getTuples().iterator(); iterator.hasNext();) {
			Tuple tuple = iterator.next();
			if (tuple.getAllowedOperation().getName().equals(request.getName())) {
				if (globalStateManager.isCurrentState(tuple.getSourceState())) {
					return tuple;
				}
			}
		}
		return null;
	}

	private void updateGlobalState(Tuple t) {
		globalStateManager.updateGlobalState(t.getTargetState());
	}

	private void sendUpdateCurrentState(Tuple t, Participant coordinationDelegateFrom) {

		for (Iterator<Participant> iterator = t.getAllowedComponentInTargetState().iterator(); iterator.hasNext();) {
			Participant coordinationDelegateTo = iterator.next();
			AdditionalMessage additionalMessage = new AdditionalMessage();
			additionalMessage.setChoreographyID(id.getChoreographyID());
			additionalMessage.setCoordinatedServiceEndpointName(id.getCoordinatedServiceEndpointName());
			additionalMessage.setCoordinationDelegateFrom(coordinationDelegateFrom.getName());
			additionalMessage.setCoordinationDelegateTo(coordinationDelegateTo.getName());
			additionalMessage.setRequest(t.getAllowedOperation().getName());
			additionalMessage.setState(t.getTargetState().getName());
			coordinationChannelManager.sendUpdateCurrentState(additionalMessage);
		}

	}

}
