package it.univaq.di.choreos.coordinationprotocol.algorithm.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationResult;
import it.univaq.di.choreos.coordinationprotocol.algorithm.GlobalStateManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Tuple;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.queue.CDQueue;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.queue.CDQueueElement;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CoordinationManagerImpl implements CoordinationDelegateManager {
	
	final Logger logger = LoggerFactory.getLogger(CoordinationManagerImpl.class);
	
	private CDQueue queue;
	private GlobalStateManager globalStateManager;
	private Coord coord;
	
	public CoordinationManagerImpl(GlobalStateManager globalStateManager, Coord coord) {
		queue = new CDQueue();
		this.globalStateManager = globalStateManager;
		this.coord = coord;
	}

	
	@Override
	public List<Participant> getAllCoordinationDelegates(AllowedOperation request) {
		logger.debug("begin getAllCoordinationDelegates. request=" + request.getName());
		EList<Participant> participants = coord.getParticipants();
		List<Participant> result = new ArrayList<Participant>();
		for (Participant participant : participants) {
			result.add(participant);
			/*
			if (checkInformation(request, participant)) {
				EList<Role> roles = participant.getAllowedComponentInSourceState();
				for (Role role : roles) {
					result.add(role.getParticipant());
				}
				break;
				
			}
			*/
		}
		logger.debug("end getAllCoordinationDelegates. participants=" + result);
		return result;
	}
	@Override
	public CoordinationResult isRequestAllowedFromS(AllowedOperation request) {
		logger.debug("begin isRequestAllowedFromS. request=" + request.getName());
		EList<Tuple> tuples = coord.getTuples();
		if ((tuples==null) || tuples.isEmpty()) {
			logger.debug("end isRequestAllowedFromS. CoordinationResult.DISCARD");
			return CoordinationResult.DISCARD;
		}
		for (Tuple tuple : tuples) {
			if (tuple.getAllowedOperation().getName().equals(request.getName())) {
				if (tuple.getSourceState().getName().equals(globalStateManager.getGlobaState().getName())) {
					logger.debug("end isRequestAllowedFromS. CoordinationResult.ALLOWED");
					return CoordinationResult.ALLOWED;					
				} else {
					logger.debug("end isRequestAllowedFromS. CoordinationResult.DISCARD");
					return CoordinationResult.DISCARD;					
				}
			}
		}
		logger.debug("end isRequestAllowedFromS. CoordinationResult.FORWARD");
		return CoordinationResult.FORWARD;
	}

	@Override
	public boolean isPrivilegeGranted(AllowedOperation request, Participant coordinatinoDelegateFrom, List<CDQueueElement> blockMessages) {
		logger.debug("begin isPrivilegeGranted. request=" + request.getName() + ", coordinatinoDelegateFrom=" + coordinatinoDelegateFrom.getName());
/*
		if ("fineco-cd".equals(coordinatinoDelegateFrom.getName())) {
			return true;
		} else {
			return false;
		}
*/		
		
		for (CDQueueElement cdQueueElement : blockMessages) {
			List<CDQueueElement> result = queue.getBlocksReceived(cdQueueElement);
			for (CDQueueElement element : result) {
				
			}
		}
		logger.debug("end isPrivilegeGranted=true");
		return true;
		
	}

	@Override
	public void updateGlobaState(AllowedOperation request) {
		logger.debug("begin updateGlobaState. request=" + request.getName());
		EList<Tuple> tuples = coord.getTuples();
		if ((tuples==null) || tuples.isEmpty()) {
			return ;
		}
		Tuple foundTuple = null;
		for (Tuple tuple : tuples) {
			foundTuple = tuple;
			/*
			if (checkInformation(request, tuple)) {
				foundTuple = tuple;
			}
			*/
		}
		if (foundTuple==null) {
			return;
		}
		globalStateManager.updateGlobalState(foundTuple.getTargetState());
		logger.debug("end updateGlobaState");
	}
	
	@Override
	public List<Participant> getAllAllowedComponentInTargetState(AllowedOperation request) {
		logger.debug("begin getAllAllowedComponentInTargetState. request=" + request.getName());
		List<Participant> result = new ArrayList<Participant>();
		EList<Tuple> tuples = coord.getTuples();
		logger.debug("getAllAllowedComponentInTargetState. tuples=" + tuples);
		if ((tuples==null) || tuples.isEmpty()) {
			return result;
		}
		Tuple foundTuple = null;
		for (Tuple tuple : tuples) {
			logger.debug("getAllAllowedComponentInTargetState. tuple =" + tuple);
			foundTuple = tuple;
			/*
			if (checkInformation(request, tuple)) {
				foundTuple = tuple;
				break;
			}
			*/
		}
		logger.debug("getAllAllowedComponentInTargetState. foundInformation=" + foundTuple);
		if (foundTuple==null) {
			return result;
		}
		
		for (Participant participant : foundTuple.getAllowedComponentInTargetState()) {
			result.add(participant);
		}
		logger.debug("end getAllAllowedComponentInTargetState. participants=" + result);
		return result;
	}	
	
	@Override
	public void addBlock2Queue(CDQueueElement cdQueueElement) {
		logger.debug("begin addBlock2Queue. cdQueueElement=" + cdQueueElement);
		queue.add(cdQueueElement);
		logger.debug("end addBlock2Queue");
	}


	@Override
	public void removeBlockFromQueue(Participant coordinationDelegateFrom, Participant coordinationDelegateTo) {
		logger.debug("begin removeBlockFromQueue. coordinationDelegateFrom=" + coordinationDelegateFrom.getName() + ", coordinationDelegateTo=" + coordinationDelegateTo.getName());
		queue.removeBlockFromQueue(coordinationDelegateFrom.getName(), coordinationDelegateTo.getName());
		logger.debug("end removeBlockFromQueue");
	}
	
	@Override
	public void removeBlocksFromQueue(String coordinationdelegatefrom, String coordinationdelegateto, State oldState) {
		queue.removeBlocksFromQueue(coordinationdelegatefrom, coordinationdelegateto, oldState.getName());
	}

	@Override
	public boolean isBefore(String coordinationDelegateFrom, String coordinationDelegateTo) {
		logger.debug("begin isBefore. coordinationDelegateFrom=" + coordinationDelegateFrom+ ", coordinationDelegateTo="+ coordinationDelegateTo);
		EList<Participant> participants = coord.getParticipants();
		if (participants==null) {
			return false;
		}
		boolean cdFromFound = false;
		for (Participant participant : participants) {
			if (participant.getName().equals(coordinationDelegateFrom)) {
				cdFromFound = true;
			}
			if (!cdFromFound && participant.getName().equals(coordinationDelegateTo)) {
				return false;
			}
			if (cdFromFound && participant.getName().equals(coordinationDelegateTo)) {
				return true;
			}
		}
		logger.debug("end isBefore");
		return cdFromFound;
		
	}

	

/*
	private boolean checkInformation(AllowedOperation request, Information information) {
		return information.getAllowedOperation().getName().equals(request.getName()) && 
				information.getSourceState().getName().equals(globalStateManager.getGlobaState().getName());
	}
*/
}
