package it.univaq.di.choreos.coordinationprotocol.algorithm.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationResult;
import it.univaq.di.choreos.coordinationprotocol.algorithm.GlobalStateManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelParser;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.queue.CDQueueElement;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

public class CoordinationManagerImplTest {
	private CoordinationManagerImpl coordinationManager;
	private AllowedOperation allowedOperation;
	private AllowedOperation notAllowedOperation;
	private State state;
	private GlobalStateManager globalStateManager;
	private Participant coordinationDelegateFrom;
	private Coord coord;
	
	@Before public void setUp() {
		state = CoordmetamodelFactory.eINSTANCE.createState();
		state.setName("0");
		globalStateManager = new GlobalStateManager(state);
		allowedOperation = CoordmetamodelFactory.eINSTANCE.createAllowedOperation();
		allowedOperation.setName("orderconfirm");
		notAllowedOperation = CoordmetamodelFactory.eINSTANCE.createAllowedOperation();
		notAllowedOperation.setName("orderconfirm1");
		String coordFileName = CoordinationManagerImplTest.class.getResource("test.coordmodel").getFile();
		coord = CoordmetamodelParser.parse(coordFileName);
		coordinationManager = new CoordinationManagerImpl(globalStateManager, coord);
		coordinationDelegateFrom = CoordmetamodelFactory.eINSTANCE.createParticipant();
		coordinationDelegateFrom.setName("fineco-cd");
	}
/*	
	@Test public void isRequestAllowedFromSTest1() {
		CoordinationResult result = coordinationManager.isRequestAllowedFromS(allowedOperation);
		Assert.assertEquals(result, CoordinationResult.ALLOWED);
	}
	@Test public void NotisRequestAllowedFromSTest2() {
		CoordinationResult result = coordinationManager.isRequestAllowedFromS(notAllowedOperation);
		Assert.assertEquals(result, CoordinationResult.FORWARD);
	}
	
	@Test public void getAllCoordinationDelegatesTest1() {
		List<Participant> result = coordinationManager.getAllCoordinationDelegates(allowedOperation);
		Assert.assertEquals(result.size(), 2);
	}
	@Test public void getAllCoordinationDelegatesTest2() {
		List<Participant> result = coordinationManager.getAllCoordinationDelegates(notAllowedOperation);
		Assert.assertEquals(result.size(), 0);
	}
	
	@Test public void isPrivilegeGrantedTest1() {
		List<CDQueueElement> blockMessages = new ArrayList<CDQueueElement>();
		
		//added to limit the scope of cdQueueElement 
		{
			int timeStamp = 1;
			CDQueueElement cdQueueElement = new CDQueueElement();
			cdQueueElement.setTimeStamp(timeStamp);
			cdQueueElement.setCoordinationDelegateFrom(coordinationDelegateFrom);
			Participant coordinationDelegateTo = CoordmetamodelFactory.eINSTANCE.createParticipant();
			coordinationDelegateTo.setName("test-cd");
			cdQueueElement.setCoordinationDelegateTo(coordinationDelegateTo);
			cdQueueElement.setState(state);
			blockMessages.add(cdQueueElement);
		}

		//added to limit the scope of cdQueueElement 
		{
			int timeStamp = 1;
			CDQueueElement cdQueueElement = new CDQueueElement();
			cdQueueElement.setTimeStamp(timeStamp);
			cdQueueElement.setCoordinationDelegateFrom(coordinationDelegateFrom);
			Participant coordinationDelegateTo = CoordmetamodelFactory.eINSTANCE.createParticipant();
			coordinationDelegateTo.setName("test-cd");
			cdQueueElement.setCoordinationDelegateTo(coordinationDelegateTo);
			cdQueueElement.setState(state);
			coordinationManager.addBlock2Queue(cdQueueElement);
		}

		boolean result = coordinationManager.isPrivilegeGranted(allowedOperation, coordinationDelegateFrom, blockMessages);
		Assert.assertTrue(result);
	}	

	@Test public void isPrivilegeGrantedTest2() {
		List<CDQueueElement> blockMessages = new ArrayList<CDQueueElement>();
		//added to limit the scope of cdQueueElement 
		{
			int timeStamp = 2;
			CDQueueElement cdQueueElement = new CDQueueElement();
			cdQueueElement.setTimeStamp(timeStamp);
			cdQueueElement.setCoordinationDelegateFrom(coordinationDelegateFrom);
			Participant coordinationDelegateTo = CoordmetamodelFactory.eINSTANCE.createParticipant();
			coordinationDelegateTo.setName("test-cd");
			cdQueueElement.setCoordinationDelegateTo(coordinationDelegateTo);
			cdQueueElement.setState(state);
			blockMessages.add(cdQueueElement);
		}

		//added to limit the scope of cdQueueElement 
		{
			int timeStamp = 1;
			CDQueueElement cdQueueElement = new CDQueueElement();
			cdQueueElement.setTimeStamp(timeStamp);
			cdQueueElement.setCoordinationDelegateFrom(coordinationDelegateFrom);
			Participant coordinationDelegateTo = CoordmetamodelFactory.eINSTANCE.createParticipant();
			coordinationDelegateTo.setName("test-cd");
			cdQueueElement.setCoordinationDelegateTo(coordinationDelegateTo);
			cdQueueElement.setState(state);
			coordinationManager.addBlock2Queue(cdQueueElement);
		}

		boolean result = coordinationManager.isPrivilegeGranted(allowedOperation, coordinationDelegateFrom, blockMessages);
		Assert.assertTrue(result);
	}
	
	@Test public void isBeforeTest1() {
		String coordinationDelegateFrom ="fineco-cd";
		String coordinationDelegateTo ="fineco-cd";
		boolean result = coordinationManager.isBefore(coordinationDelegateFrom, coordinationDelegateTo);
		Assert.assertTrue(result);
	}

	@Test public void isBeforeTest2() {
		String coordinationDelegateFrom ="fineco-cd";
		String coordinationDelegateTo ="ubs-cd";
		boolean result = coordinationManager.isBefore(coordinationDelegateFrom, coordinationDelegateTo);
		Assert.assertTrue(result);
	}

	@Test public void isBeforeTest3() {
		String coordinationDelegateFrom ="ubs-cd";
		String coordinationDelegateTo ="fineco-cd";
		boolean result = coordinationManager.isBefore(coordinationDelegateFrom, coordinationDelegateTo);
		Assert.assertFalse(result);
	}	

	@Test public void isBeforeTest4() {
		String coordinationDelegateFrom ="test1-cd";
		String coordinationDelegateTo ="test2-cd";
		boolean result = coordinationManager.isBefore(coordinationDelegateTo, coordinationDelegateFrom);
		Assert.assertFalse(result);
	}		
*/
/*
 * 	CoordinationResult isRequestAllowedFromS(AllowedOperation request);
	List<Participant> getAllCoordinationDelegates(AllowedOperation request);
	boolean isPrivilegeGranted(AllowedOperation request, Participant coordinatinoDelegateFrom, List<CDQueueElement> blockMessages);
	boolean isBefore(String coordinationDelegateFrom, String coordinationDelegateTo);
		
	void updateGlobaState(AllowedOperation request);
	List<Participant> getAllAllowedComponentInTargetState(AllowedOperation request);
	void addBlock2Queue(CDQueueElement cdQueueElement);
	void removeBlockFromQueue(Participant coordinationDelegateFrom, Participant coordinationDelegateTo);
	void removeBlocksFromQueue(String coordinationdelegatefrom, String coordinationdelegateto, State oldState);


 * */	
}
