package it.univaq.di.choreos.ELTS;

import it.univaq.di.choreos.ELTS.impl.ELTSPackageImpl;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Date;

import org.apache.commons.io.FileUtils;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

public class ELTSMetamodelParser {
	static {
		ELTSPackageImpl.init();
	}

	public static Model parse(String modelFileName) {
		URI fileURI = URI.createFileURI(modelFileName);
		Resource resource = new XMIResourceFactoryImpl().createResource(fileURI);
		try {
			resource.load(null);
			Model elts = (Model) resource.getContents().get(0);
			return elts;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	public static Model getEltsModel(String eltsContent) {
		Model elts = null;
		String tempFolder = createTempFolder();
		try {
			File eltsFile = new File(tempFolder + File.separator + "eltstemp.elts");
			FileUtils.writeStringToFile(eltsFile, eltsContent);
			elts = parse(eltsFile.getAbsolutePath());
			FileUtils.deleteQuietly(new File(tempFolder));
		} catch (IOException e) {
			e.printStackTrace();
		} 
		return elts;
	}
	
	public static String getEltsContent(Model cltsModel) {
		String tempFolder = createTempFolder();
		File tempElts = new File(tempFolder+File.separator + "eltstemp.elts");
		// create resource from the Model
		URI fileUriTempModelNormalized = URI.createFileURI(tempElts.getAbsolutePath());
		Resource resourceModelNormalized = new XMIResourceFactoryImpl().createResource(fileUriTempModelNormalized);
		// add model in model resourceModel
		resourceModelNormalized.getContents().add(cltsModel);
		String eltsContent = "";
		try {
			// save new ELTS model
			resourceModelNormalized.save(Collections.EMPTY_MAP);
			eltsContent = FileUtils.readFileToString(tempElts);
			FileUtils.deleteQuietly(new File(tempFolder));
		} catch (IOException e) {
			e.printStackTrace();
		}
		return eltsContent;
	}
	
	
	
	private static String createTempFolder(){
		String tempFolder = getTempFolder();
		File tempFolderDir = new File(tempFolder);
		try {
			FileUtils.forceMkdir(tempFolderDir);
			return tempFolder;
		} catch (IOException e) {
			e.printStackTrace();
		}finally {
			try {
				FileUtils.deleteDirectory(tempFolderDir);
			} catch (IOException e) {
			}
		}
		return "";
	}

	private static String getTempFolder() {
		return FileUtils.getTempDirectoryPath() + File.separator + getRandomDir() + File.separator;
	}

	private static String getRandomDir() {
		return new Date().getTime() + "";
	}


}