package it.univaq.di.choreos.elts.strongsimulation;

import it.univaq.di.choreos.ELTS.ActivityLoop;
import it.univaq.di.choreos.ELTS.ActivityLoop2State;
import it.univaq.di.choreos.ELTS.Alternative;
import it.univaq.di.choreos.ELTS.Alternative2State;
import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.FinalState;
import it.univaq.di.choreos.ELTS.Fork;
import it.univaq.di.choreos.ELTS.Fork2State;
import it.univaq.di.choreos.ELTS.InitialState;
import it.univaq.di.choreos.ELTS.InitialState2State;
import it.univaq.di.choreos.ELTS.Join;
import it.univaq.di.choreos.ELTS.Join2State;
import it.univaq.di.choreos.ELTS.LabeledState2State;
import it.univaq.di.choreos.ELTS.LabeledTransition;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.NamedElement;
import it.univaq.di.choreos.ELTS.Operation;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2ActivityLoop;
import it.univaq.di.choreos.ELTS.State2Alternative;
import it.univaq.di.choreos.ELTS.State2FinalState;
import it.univaq.di.choreos.ELTS.State2Fork;
import it.univaq.di.choreos.ELTS.State2Join;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.ELTS.Transition;

import java.util.List;

public interface ELTSModel {
	//void check();
	/**
	 * 
	 * @return the Model
	 */
	Model getModel();
	
	/**
	 * 
	 * @param model model
	 * @param operationName operation name
	 * @return operation, null otherwise
	 */
	Operation getOperation(String operationName);

	Model getModelNormalized();
	
	List<ActivityLoop> getActivityLoops();
	List<Alternative> getAlternatives();
	List<Fork> getForks();
	List<Join> getJoins();
	List<State> getStates();
	FinalState getFinalState();
	InitialState getInitialState();
	
	
	void insertState (State state);
	void insertStates(List<State> states);
	void insertFork (Fork fork);
	void insertJoin (Join join);
	void insertAlternative (Alternative alternative);
	void insertActivityLoop (ActivityLoop activityLoop);
	
	
	void removeState (State state);
	void removeStates (List<State> states);
	void removeFork (Fork fork);
	void removeForks (List<Fork> forks);
	void removeJoin (Join join);
	void removeJoins (List<Join> joins);
	void removeAlternative (Alternative alternative);
	void removeAlternatives (List<Alternative> alternatives);
	void removeActivityLoop (ActivityLoop activityLoop);
	void removeActivityLoops (List<ActivityLoop> activityLoops);
	
	
	List<LabeledTransition> getLabeledTransitions();
	List<EpsilonTransition> getEpsilonTransitions();
	
	
	void insertLabeledTransition(LabeledTransition labeledTransition);
	void insertLabeledState2State(LabeledState2State labeledState2State);
	void insertEpsilonTransition(EpsilonTransition epsilonTransition);
	void insertJoin2State(Join2State join2State);
	void insertState2Fork(State2Fork state2Fork);
	void insertFork2State(Fork2State fork2State);
	void insertState2Join(State2Join state2Join);
	void insertInitialState2State(InitialState2State initialState2State);
	void insertState2FinalState(State2FinalState state2FinalState);
	void insertState2Alternative(State2Alternative state2Alternative);
	void insertAlternative2State(Alternative2State alternative2State);
	void insertState2ActivityLoop(State2ActivityLoop state2ActivityLoop);
	void insertActivityLoop2State(ActivityLoop2State activityLoop2State);
	void insertState2State(State2State state2State);
	
	
	void removeLabeledState2State(LabeledState2State labeledState2State);
	void removeLabeledTransition(LabeledTransition labeledTransition);	
	void removeLabeledTransitions(List<LabeledTransition> labeledTransitions);
	void removeJoin2State(Join2State join2State);
	void removeState2Fork(State2Fork state2Fork);
	void removeFork2State(Fork2State fork2State);
	void removeState2Join(State2Join state2Join);
	void removeInitialState2State(InitialState2State initialState2State);
	void removeState2FinalState(State2FinalState state2FinalState);
	void removeState2Alternative(State2Alternative state2Alternative);
	void removeAlternative2State(Alternative2State alternative2State);
	void removeState2ActivityLoop(State2ActivityLoop state2ActivityLoop);
	void removeActivityLoop2State(ActivityLoop2State activityLoop2State);
	void removeState2State(State2State state2State);
	void removeEpsilonTransitions(List<EpsilonTransition> epsilonTransitions);
	void removeEpsilonTransition(EpsilonTransition epsilonTransition);
	
	
	void editSourceLabeledState2State(LabeledState2State labeledState2State, State state);
	void editSourceJoin2State(Join2State join2State, Join join);
	void editSourceState2Fork(State2Fork state2Fork, State state);
	void editSourceFork2State(Fork2State fork2State, Fork fork);
	void editSourceState2Join(State2Join state2Join, State state);
	void editSourceInitialState2State(InitialState2State initialState2State, InitialState initialState);
	void editSourceState2FinalState(State2FinalState state2FinalState, State state);
	void editSourceState2Alternative(State2Alternative state2Alternative, State state);
	void editSourceAlternative2State(Alternative2State alternative2State, Alternative alternative);
	void editSourceState2ActivityLoop(State2ActivityLoop state2ActivityLoop, State state);
	void editSourceActivityLoop2State(ActivityLoop2State activityLoop2State, ActivityLoop activityLoop);
	void editSourceState2State(State2State state2State, State state);
	
	
	
	
	
	
	

	/**
	 * 
	 * @param model model
	 * @return list of transitions from the Initial State to other State
	 */
	List<InitialState2State> getAllInitial2StateTransitions();

	/**
	 * 
	 * @param model model
	 * @param state state to check
	 * @return a {@link Boolean} value that indicates if the state has a transition to the Final State
	 * 
	 */
	boolean existTransitionToFinalState(State state);

	/**
	 * 
	 * @param model model
	 * @param stateSource state of the model
	 * @return the list of labeled state to state transitions from state
	 */
	List<LabeledState2State> getAllState2StateTransitionsFromStateSource(State stateSource);

	/**
	 * 
	 * @param model model
	 * @param stateSource state of the model
	 * @return the list of epsilon state to state transitions from state
	 */
	List<State2State> getAllEpsilonState2StateTransitionsFromStateSource(State stateSource);

	/**
	 * 
	 * @param model model
	 * @return all Epsilon State 2 State transitions
	 */
	List<State2State> getAllEpsilonState2StateTransitions();

	/**
	 * 
	 * @param model model
	 * @param transition transition to check
	 * @param state state to start check
	 * @return the State to State Transitions allowable
	 */
	List<LabeledState2State> getAllState2StateTransitionsAllowable(LabeledState2State transition, State state);

	/**
	 * 
	 * @param model model
	 * @param state state
	 * @return list of Forks from state
	 */
	List<Fork> getAllForksFromState(State state);

	/**
	 * 
	 * @param model model
	 * @param fork fork
	 * @return list of start states by fork
	 */
	List<State> getAllStartStatesByFork(Fork fork);

	

	/**
	 * 
	 * @param model model
	 * @param state state
	 * @return the join connected by state, null otherwise
	 */

	Join getJoinConnectedByState(State state);

	/**
	 * 
	 * @param model model
	 * @param join join
	 * @return NamedElement connected by join, null otherwise 
	 */
	NamedElement getNamedElementConnectedByJoin(Join join);

	/**
	 * 
	 * @param model model
	 * @param join join
	 * @return EpsilonTransition from join, null otherwise
	 */
	EpsilonTransition getTransitionFromJoin(Join join);

	/**
	 * 
	 * @param model model
	 * @param fork fork
	 * @return NamedElement connected to fork, null otherwise 
	 */
	NamedElement getNamedElementConnectedToFork(Fork fork);

	/**
	 * 
	 * @param model model
	 * @param fork fork
	 * @return EpsilonTransition to fork, null otherwise 
	 */
	EpsilonTransition getTransitionToFork(Fork fork);

	/**
	 * 
	 * @param model model
	 * @param fork fork
	 * @return all EpsilonTransitions from fork
	 */
	List<EpsilonTransition> getAllTransitionsFromFork(Fork fork);

	/**
	 * 
	 * @param model model
	 * @param join join
	 * @return all EpsilonTransitions to join
	 */
	List<EpsilonTransition> getAllTransitionsToJoin(Join join);

	/**
	 * 
	 * @param model model
	 * @param state state
	 * @return all transitions to state
	 */
	List<Transition> getAllTransitionsToState(State state);

	/**
	 * 
	 * @param model model
	 * @param state state
	 * @return all transitions to state
	 */
	List<Transition> getAllTransitionsFromState(State state);

	/**
	 * 
	 * @param model model
	 * @param state state
	 * @param join join
	 * @return the State2Join transition from state and join sources, null otherwise
	 */
	State2Join getState2JoinTransition(State state, Join join);

	/**
	 * 
	 * @param model model
	 * @param state state
	 * @return the State2FinalState Transition from state source, null otherwise
	 */
	State2FinalState getState2Final(State state);

	/**
	 * 
	 * @param model model
	 * @param fork fork
	 * @param state state
	 * @return the Fork2State transition from fork and state sources, null otherwise
	 */
	Fork2State getFork2StateTransition(Fork fork, State state);

	/**
	 * 
	 * @param model model
	 * @param activityLoop activity loop
	 * @return all transitions to activity loop
	 */
	List<EpsilonTransition> getAllTransitionsToActivityLoop(ActivityLoop activityLoop);

	/**
	 * 
	 * @param model model
	 * @param activityLoop activity loop
	 * @return all epsilon transitions from activity loop
	 */
	List<EpsilonTransition> getAllTransitionsFromActivityLoop(ActivityLoop activityLoop);

	/**
	 * 
	 * @param model model
	 * @param alternative alternative
	 * @return all epsilon transitions to alternative
	 */
	List<EpsilonTransition> getAllTransitionsToAlternative(Alternative alternative);

	/**
	 * 
	 * @param model model
	 * @param alternative alternative
	 * @return all epsilon transitions from alternative
	 */
	List<EpsilonTransition> getAllTransitionsFromAlternative(Alternative alternative);

	/**
	 * 
	 * @param model model
	 * @param transition transition to check
	 * @return a value that indicates whether the transition already exists
	 */
	boolean existTransition(Transition transition);

	/**
	 * 
	 * @param model model
	 */
	void removeState2StateWithSourceAndTargetEquals();

	/**
	 * 
	 * @param model model
	 */
	void removeStateAndTransitionIsolated();

	/**
	 * 
	 * @param model model to be copied 
	 * @return a new model copied
	 */
	Model copyModel();
	/**
	 * 
	 * @param fullPathDestinationModel to save model
	 * @return a value that indicates whether the save was successful 
	 */
	boolean saveModel(String fullPathDestinationModel);

}