package it.univaq.di.choreos.elts.strongsimulation;

import it.univaq.di.choreos.ELTS.EpsilonTransition;
import it.univaq.di.choreos.ELTS.FinalState;
import it.univaq.di.choreos.ELTS.Fork;
import it.univaq.di.choreos.ELTS.Fork2State;
import it.univaq.di.choreos.ELTS.Join;
import it.univaq.di.choreos.ELTS.Join2State;
import it.univaq.di.choreos.ELTS.LabeledState2State;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.NamedElement;
import it.univaq.di.choreos.ELTS.Operation;
import it.univaq.di.choreos.ELTS.OperationType;
import it.univaq.di.choreos.ELTS.State;
import it.univaq.di.choreos.ELTS.State2FinalState;
import it.univaq.di.choreos.ELTS.State2Fork;
import it.univaq.di.choreos.ELTS.State2Join;
import it.univaq.di.choreos.ELTS.State2State;
import it.univaq.di.choreos.ELTS.Transition;
import it.univaq.di.choreos.elts.strongsimulation.impl.ELTSManagerImpl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 
 * @author Alexander Perucci
 *
 */
public class RemoveForkObjects {
	
	private ELTSModel modelManager;
	
	public RemoveForkObjects(ELTSModel modelManager){
		this.modelManager=modelManager;
		removeForkObjects();
	}
	/**
	 * 
	 * @return the model without Forks
	 */
	public Model getModelWithoutForks() {
		return modelManager.getModel();
	}

	
	
	
	/**
	 * Remove Forks
	 */
	private void removeForkObjects() {		
		// pre-processing model
		List<Map<String, Object>> resultsPreProcessing = preProcessingModel();
		// there isn't fork to analyze then it returns model
		if (resultsPreProcessing.size() == 0){
			return;
		}
		// there is fork to analyze
		for (Map<String, Object> resultPreProcessing : resultsPreProcessing) {
			if (Boolean.valueOf(resultPreProcessing.get("analyze").toString())) {

				// analyze fork
				List<Map<String, Object>> resultsAnalyzeFork = analyzeFork((Fork) resultPreProcessing.get("fork"));
				
				// build interleaving
				buildInterleaving(resultsAnalyzeFork, (Fork) resultPreProcessing.get("fork"));
			}
		}


		// recursion function to ensure that there aren't other fork to remove
		removeForkObjects();
	}


	// PRE-PROCESSING SECTION: check if forks have not other nested fork
	/**
	 * 
	 * @return list of Map where each map contains this value: fork, analyze
	 */
	private List<Map<String, Object>> preProcessingModel() {
		List<Map<String, Object>> forksAnalyze = new ArrayList<Map<String, Object>>();
		// get all fork in model
		List<Fork> forks = modelManager.getForks();
		for (Fork fork : forks) {
			forksAnalyze.add(preProcessingModelFromFork( fork));
		}

		return forksAnalyze;
	}

	/**
	 * 
	 * @param fork fork to analyzed
	 * @return map object
	 */
	private Map<String, Object> preProcessingModelFromFork( Fork fork) {

		Map<String, Object> mapFork = new HashMap<String, Object>();
		// set mapFork
		mapFork.put("fork", fork);
		mapFork.put("analyze", true);


		// get all state connected by fork
		List<State> startStatesConnectedByFork = modelManager.getAllStartStatesByFork( fork);
			for (State startStateFork : startStatesConnectedByFork) {
				// pre-processing model from start state
				preProcessingModelFromState( mapFork, new ArrayList<State>(), startStateFork);
				// check if fork is not to be analyze
				if (!Boolean.valueOf(mapFork.get("analyze").toString()))
					return mapFork;
			}
		

		return mapFork;
	}

	/**
	 * 
	 * @param mapFork map of the fork
	 * @param statesAnalyzedInFork states analyzed in fork
	 * @param state state to analyzed
	 */
	private void preProcessingModelFromState( Map<String, Object> mapFork, List<State> statesAnalyzedInFork, State state) {
		// check if state is already analyzed
		if (!statesAnalyzedInFork.contains(state))
			statesAnalyzedInFork.add(state);
		else
			return;

		// check if state is connected to fork
		List<Fork> forks = modelManager.getAllForksFromState( state);
		if (forks.size() != 0) {
			mapFork.put("analyze", false);
			return;

		}

		// get all state2state transitions from source state
		for (LabeledState2State state2StateTransition : modelManager.getAllState2StateTransitionsFromStateSource( state)) {
			preProcessingModelFromState( mapFork, statesAnalyzedInFork, state2StateTransition.getTarget());
			if (!Boolean.valueOf(mapFork.get("analyze").toString()))
				return;
		}
		
		//get all epsilon state2state transitions from source state
		for (State2State epsilonState2State : modelManager.getAllEpsilonState2StateTransitionsFromStateSource( state)) {
			preProcessingModelFromState( mapFork, statesAnalyzedInFork, epsilonState2State.getTarget());
			if (!Boolean.valueOf(mapFork.get("analyze").toString()))
				return;
		}
		
		
		
		return;
	}

	// ANALYZE FORK SECTION: get representation of each fork tracks

	/**
	 * 
	 * @param fork fork
	 * @return list of Map where each map contains this value: representation, lastElement
	 */
	private List<Map<String, Object>> analyzeFork(Fork fork) {
		//list of all fork tracks
		 List<Map<String, Object>> allTracks =new ArrayList<Map<String,Object>>();
		// list of states connected to fork
		List<State> statesConnectedAfterFork = modelManager.getAllStartStatesByFork( fork);
		// add in representationsModel the representation of each track of fork
		for (State stateConnectedAfterFork: statesConnectedAfterFork) {
			allTracks.add(analyzeForkFromStartState( stateConnectedAfterFork));
		}
		
		return allTracks;
	}

	/**
	 * 
	 * @param startState start state fork track
	 * @return the representation of track
	 */

	private Map<String, Object> analyzeForkFromStartState( State startState) {
		// representationsModel contains the representation of fork track
		RepresentationTrackFork representationTrack = new RepresentationTrackFork();
		// object that representation of track
		Map <String, Object> track =new HashMap<String, Object>();
		track.put("representation", representationTrack);
		
		// add in representation track the state
		representationTrack.getStates().add(startState);
		// add in representation track the start state
		representationTrack.setStartState(startState);
		
		// each state visited is insert into list, help to end of recursion
		NamedElement lastElement= analyzeForkFromState( new ArrayList<State>(), representationTrack, startState);
		track.put("lastElement", lastElement);

		return track;
	}

	/**
	 * 
	 * @param representation representation of track
	 * @param statesAnalyzedInFork state analyze
	 * @param state state to check
	 */
	private NamedElement analyzeForkFromState( List<State> statesAnalyzedInFork, RepresentationTrackFork representation, State state) {
		NamedElement lastElement=null;
		// check if state is already analyzed
		if (!statesAnalyzedInFork.contains(state))
			statesAnalyzedInFork.add(state);
		else
			return lastElement;

		// add state in representation
		if (representation.getState(state.getName()) == null) {
			representation.getStates().add(state);
		}
		
		// check if state is a endState, means that state is connected to join
		// or finalState
		Join join = modelManager.getJoinConnectedByState( state);
		if (join != null) {
			representation.setEndState(state);
			lastElement = join;
		} else if (modelManager.existTransitionToFinalState( state)) {
			representation.setEndState(state);
			lastElement = modelManager.getFinalState();

		}
	

		// get all State2State (labeled) transitions
		List<LabeledState2State> state2StateTransition = modelManager.getAllState2StateTransitionsFromStateSource( state);
		for (LabeledState2State state2StateTransitionSelected : state2StateTransition) {
			// navigation model for each transition State2State
			NamedElement elementReturned=analyzeForkFromState( statesAnalyzedInFork, representation, state2StateTransitionSelected.getTarget());
			if (elementReturned!=null)
				lastElement =elementReturned;
			// add State2StateTransition in representation
			representation.getState2StateTransitions().add(state2StateTransitionSelected);
		}
		
		// get all State2State (epsilon) transitions
		for (State2State epsilonState2State : modelManager.getAllEpsilonState2StateTransitionsFromStateSource( state)) {
			// navigation model for each transition State2State
			NamedElement elementReturned=analyzeForkFromState( statesAnalyzedInFork, representation, epsilonState2State.getTarget());
			if (elementReturned!=null)
				lastElement =elementReturned;
			// add State2StateTransition in representation
			representation.getState2StateTransitions().add(epsilonState2State);
		}
			
		return lastElement;
	}


	
		
	
	
	// BUILD INTERLEAVING SECTION: build interleaving for fork selected
	/**
	 * 
	 * @param resultsAnalyzeFork results of analyze fork
	 * @param fork fork
	 */
	private void buildInterleaving( List<Map<String, Object>> resultsAnalyzeFork, Fork fork) {
		// get all joins achieved
		List<Join> joinsReached = new ArrayList<Join>();
		for (Map<String, Object> resultAnalyzeFork : resultsAnalyzeFork) {
			NamedElement element = (NamedElement) resultAnalyzeFork.get("lastElement");
			if (element instanceof Join) {
				if (!joinsReached.contains(element)) {
					joinsReached.add((Join) element);
				}
			}
		}

		//check if joins achieved
		if (joinsReached.size() > 0) {
			buildInterleavingWithJoins( resultsAnalyzeFork, fork, joinsReached);
		} else {
			//all traces ending the final state
			buildInterleavingWithoutJoins( resultsAnalyzeFork, fork);
		}

	}
	
	/**
	 * 
	 * @param resultsAnalyzeFork results of analyze fork
	 * @param fork fork
	 * @param joins joins
	 */
	private void buildInterleavingWithJoins( List<Map<String, Object>> resultsAnalyzeFork, Fork fork, List<Join> joins){
		for (Join joinReached : joins) {
			// get all the tracks that end in selected join
			List<RepresentationTrackFork> tracksReachedJoin = new ArrayList<RepresentationTrackFork>();
			for (Map<String, Object> resultAnalyzeFork : resultsAnalyzeFork) {
				NamedElement element = (NamedElement) resultAnalyzeFork.get("lastElement");
				if (element instanceof Join) {
					if (element.getName().compareTo(joinReached.getName()) == 0) {
						tracksReachedJoin.add((RepresentationTrackFork) resultAnalyzeFork.get("representation"));
					}
				}
			}
			
			//make a representation of interleaving
			RepresentationTrackFork representationAfterInterleaving = new RepresentationTrackFork();
			representationAfterInterleaving = tracksReachedJoin.get(0);	
			// build interleaving
			if (tracksReachedJoin.size() > 1) {
				for (int i = 0; i < tracksReachedJoin.size() - 1; i++) {
					// interleaving representation
					representationAfterInterleaving = buildInterleavingTracks(representationAfterInterleaving, tracksReachedJoin.get(i + 1));
					// restore name state
					representationAfterInterleaving.adjustsNameState();
				}
				//remove elements into fork
				editModelRemoveElements( tracksReachedJoin);
				// add new elements
				editModelAddElements( representationAfterInterleaving);
			}
			
			
			//check if to remove join
			if(tracksReachedJoin.size()==(modelManager.getAllTransitionsToJoin( joinReached)).size()){
				//edit  remove join
				editModelRemoveJoin( joinReached, representationAfterInterleaving.getEndState());
			}
			else{
				//edit  without removing join
				editModelWithoutRemoveJoin( tracksReachedJoin, joinReached, representationAfterInterleaving.getEndState());
			}
			
			//check if to remove fork
			if ( tracksReachedJoin.size()==(modelManager.getAllTransitionsFromFork( fork)).size()){
				// edit  remove fork
				editModelRemoveFork( fork, representationAfterInterleaving.getStartState());
			}else{
				//edit  without removing fork
				editModelWithoutRemoveFork( tracksReachedJoin, fork, representationAfterInterleaving.getStartState());
			}
		}
	}
	
	/**
	 * 
	 * @param resultsAnalyzeFork results of analyze fork
	 * @param fork fork
	 */
	private void buildInterleavingWithoutJoins( List<Map<String, Object>> resultsAnalyzeFork, Fork fork){
		
		// get all the tracks that end in final state
		List<RepresentationTrackFork> tracksReachedFinalState = new ArrayList<RepresentationTrackFork>();
		for (Map<String, Object> resultAnalyzeFork : resultsAnalyzeFork) {
			NamedElement element = (NamedElement) resultAnalyzeFork.get("lastElement");
			if (element instanceof FinalState) {
				if (element.getName().compareTo(modelManager.getFinalState().getName()) == 0) {
					tracksReachedFinalState.add((RepresentationTrackFork) resultAnalyzeFork.get("representation"));
				}
			}
		}
		//make a representation of interleaving
		RepresentationTrackFork representationAfterInterleaving = new RepresentationTrackFork();
		
		representationAfterInterleaving = tracksReachedFinalState.get(0);	
		
		// build interleaving
		if (tracksReachedFinalState.size() > 1) {
			for (int i = 0; i < tracksReachedFinalState.size() - 1; i++) {
				// interleaving representation
				representationAfterInterleaving = buildInterleavingTracks(representationAfterInterleaving, tracksReachedFinalState.get(i + 1));
				// restore name state
				representationAfterInterleaving.adjustsNameState();
			}
			
			//remove elements into fork
			editModelRemoveElements( tracksReachedFinalState);
			// add new elements
			editModelAddElements( representationAfterInterleaving);
			
		}
			//edit  remove fork
			editModelRemoveFork( fork, representationAfterInterleaving.getStartState());
			//edit model
			editModelWithoutRemoveFinalState( tracksReachedFinalState, representationAfterInterleaving.getEndState());
	}
	
	/**
	 * 
	 * @param firstTrackRepresentation representation of first track
	 * @param secondTrackRepresentation representation of second track
	 * @return representation built after interleaving
	 */
	private RepresentationTrackFork buildInterleavingTracks(RepresentationTrackFork firstTrackRepresentation, RepresentationTrackFork secondTrackRepresentation) {
		// create object representation model
		RepresentationTrackFork interleavingRepresentation = new RepresentationTrackFork();
		
		// interleaving the States
		List<State> states = buildInterleavingStates(firstTrackRepresentation.getStates(), secondTrackRepresentation.getStates());
		interleavingRepresentation.setStates(states);
		
		

		// interleaving the Start State
		State newStartState = interleavingRepresentation.getState(firstTrackRepresentation.getStartState().getName() + "/,/" + secondTrackRepresentation.getStartState().getName());
		interleavingRepresentation.setStartState(newStartState);

		// interleaving the End States
		State newEndStates = interleavingRepresentation.getState(firstTrackRepresentation.getEndState().getName()+"/,/"+ secondTrackRepresentation.getEndState().getName());
		interleavingRepresentation.setEndState(newEndStates);


		// interleaving the State2State Transition
		List<Transition> state2StateTransitions = buildInterleavingState2StateTransitions(firstTrackRepresentation, secondTrackRepresentation, interleavingRepresentation);
		interleavingRepresentation.setState2StateTransitions(state2StateTransitions);
		

		return interleavingRepresentation;
	}

	/**
	 * 
	 * @param firstTrackStates states list of first track
	 * @param secondTrackStates states list of second track
	 * @return states built after interleaving 
	 */
	private List<State> buildInterleavingStates(List<State> firstTrackStates, List<State> secondTrackStates) {
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
		
		List<State> states = new ArrayList<State>();
		for (State stateFirst : firstTrackStates) {
			for (State stateSecond : secondTrackStates) {
				State newState = cltsFactoryManager.createState(stateFirst.getName() + "/,/" + stateSecond.getName());
				states.add(newState);
			}
		}

		return states;
	}

	/**
	 * @param firstTrackRepresentation representation of first track
	 * @param secondTrackRepresentation representation of second track
	 * @param interleavingRepresentation representation after interleaving
	 * @return state to state transitions built after interleaving 
	 */
	private List<Transition> buildInterleavingState2StateTransitions( RepresentationTrackFork firstTrackRepresentation, RepresentationTrackFork secondTrackRepresentation, RepresentationTrackFork interleavingRepresentation) {
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
		
		List<Transition> state2StateTransitions = new ArrayList<Transition>();
		// analyze State2State Transition in first representation
		for (State stateSecondRepresentation : secondTrackRepresentation.getStates()) {
			for (Transition state2StateTransition : firstTrackRepresentation.getState2StateTransitions()) {
				if (state2StateTransition instanceof LabeledState2State){
					List<State> statesSource = interleavingRepresentation.getStatesWithSecondStateHold(((LabeledState2State)state2StateTransition).getSource(), stateSecondRepresentation);
					List<State> statesTarget = interleavingRepresentation.getStatesWithSecondStateHold(((LabeledState2State)state2StateTransition).getTarget(), stateSecondRepresentation);
					for (State stateSource : statesSource) {
						for (State stateTarget : statesTarget) {
							// create Operation
							Operation newOperation = modelManager.getOperation( ((LabeledState2State)state2StateTransition).getLabel().getName());
							OperationType operationType =  ((LabeledState2State)state2StateTransition).getOperationType();
							// create State2StateTransition
							LabeledState2State newState2StateTransition = cltsFactoryManager.createLabeledState2State(stateSource, stateTarget, operationType, newOperation);
							// add State2StateTransition in representation
							state2StateTransitions.add(newState2StateTransition);
						}
					}

				} else if (state2StateTransition instanceof State2State){
					List<State> statesSource = interleavingRepresentation.getStatesWithSecondStateHold(((State2State)state2StateTransition).getSource(), stateSecondRepresentation);
					List<State> statesTarget = interleavingRepresentation.getStatesWithSecondStateHold(((State2State)state2StateTransition).getTarget(), stateSecondRepresentation);
					for (State stateSource : statesSource) {
						for (State stateTarget : statesTarget) {
							// create EpsilonState2State
							State2State newEpsilonState2StateTransition = cltsFactoryManager.createState2State(stateSource, stateTarget);
							// add EpsilonState2State in representation
							state2StateTransitions.add(newEpsilonState2StateTransition);
						}
					}
				}
			}

		}	
		
		// analyze State2State Transition in second representation
		for (State stateFirstRepresentation : firstTrackRepresentation.getStates()) {
			for (Transition state2StateTransition : secondTrackRepresentation.getState2StateTransitions()) {
				if (state2StateTransition instanceof LabeledState2State){
					List<State> statesSource = interleavingRepresentation.getStatesWithFirstStateHold(((LabeledState2State)state2StateTransition).getSource(), stateFirstRepresentation);
					List<State> statesTarget = interleavingRepresentation.getStatesWithFirstStateHold(((LabeledState2State)state2StateTransition).getTarget(), stateFirstRepresentation);
					for (State stateSource : statesSource) {
						for (State stateTarget : statesTarget) {
							// create Operation
							Operation newOperation = modelManager.getOperation(((LabeledState2State)state2StateTransition).getLabel().getName());
							OperationType newoperationType = ((LabeledState2State)state2StateTransition).getOperationType();
							// create State2StateTransition
							LabeledState2State newState2StateTransition = cltsFactoryManager.createLabeledState2State(stateSource, stateTarget, newoperationType, newOperation);
					
							// add State2StateTransition in representation
							state2StateTransitions.add(newState2StateTransition);
						}
					}

				}else if (state2StateTransition instanceof State2State){
					List<State> statesSource = interleavingRepresentation.getStatesWithFirstStateHold(((State2State)state2StateTransition).getSource(), stateFirstRepresentation);
					List<State> statesTarget = interleavingRepresentation.getStatesWithFirstStateHold(((State2State)state2StateTransition).getTarget(), stateFirstRepresentation);
					for (State stateSource : statesSource) {
						for (State stateTarget : statesTarget) {
							// create EpsilonState2State
							State2State newEpsilonState2StateTransition = cltsFactoryManager.createState2State(stateSource, stateTarget);
							// add EpsilonState2State in representation
							state2StateTransitions.add(newEpsilonState2StateTransition);
						}
					}
				}
			}

		}

		return state2StateTransitions;
	}



	// EDIT MODEL SECTION: remove and add elements in model
	/**
	 * 
	 * @param fork fork
	 * @param startState start state after interleaving
	 */
	private void editModelRemoveFork (Fork fork, State startState){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
		
		//get element connected to fork
		NamedElement elementBeforeFork = modelManager.getNamedElementConnectedToFork(fork);
		//get transition from elementBeforeFork to Fork
		EpsilonTransition transitionBeforeFork = modelManager.getTransitionToFork(fork);
		//get all transitions from fork
		List<EpsilonTransition> transitionsAfterFork =modelManager.getAllTransitionsFromFork(fork);
			
		//remove transition to fork
		modelManager.removeEpsilonTransition(transitionBeforeFork);
		//remove transition from fork
		modelManager.removeEpsilonTransitions(transitionsAfterFork);
		//remove fork
		modelManager.removeFork(fork);
		
		
		//connected elementBeforeFork to startState
		if (elementBeforeFork instanceof Fork){
			Fork2State fs=cltsFactoryManager.createFork2State((Fork)elementBeforeFork, startState);
			modelManager.insertFork2State(fs);
		}else if (elementBeforeFork instanceof Join){
			Join2State js=cltsFactoryManager.createJoin2State((Join)elementBeforeFork, startState);
			modelManager.insertJoin2State(js);
		}else if (elementBeforeFork instanceof State){
			State2State ess=cltsFactoryManager.createState2State((State)elementBeforeFork, startState);
			modelManager.insertState2State(ess);
		}  
			
				
				
		
	}
	
	/**
	 * 
	 * @param tracksRepresentation representation of all track from fork
	 * @param fork fork
	 * @param startState start state after interleaving
	 */
	private void editModelWithoutRemoveFork ( List<RepresentationTrackFork> tracksRepresentation, Fork fork, State startState ){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
				
		for (RepresentationTrackFork trackRepresentation : tracksRepresentation) {
			//remove Fork2StateTransition
			modelManager.removeFork2State(modelManager.getFork2StateTransition(fork, trackRepresentation.getStartState()));
		}
		// connected fork to start state
		Fork2State fs = cltsFactoryManager.createFork2State(fork, startState);
		modelManager.insertFork2State(fs);
	}
	
	/**
	 * 
	 * @param join join
	 * @param endState end state after interleaving
	 */
	private void editModelRemoveJoin ( Join join, State endState){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
				
		//get element connected by join
		NamedElement elementAfterJoin = modelManager.getNamedElementConnectedByJoin(join);
		// get transition from join to elementAfterJoin
		EpsilonTransition transitionAfterJoin = modelManager.getTransitionFromJoin(join);
		//get all transitions to join
		List<EpsilonTransition> transitionsBeforeJoin =modelManager.getAllTransitionsToJoin(join);
		
		//remove transition from join
		modelManager.removeEpsilonTransition(transitionAfterJoin);		
		//remove transition to join
		modelManager.removeEpsilonTransitions(transitionsBeforeJoin);
		//remove join
		modelManager.removeJoin(join);
		
		//connected end state to elementAfterJoin
		if (elementAfterJoin instanceof Fork){
			State2Fork sf=cltsFactoryManager.createState2Fork(endState, (Fork)elementAfterJoin);
			modelManager.insertState2Fork(sf);
		}else if (elementAfterJoin instanceof Join){
			State2Join sj=cltsFactoryManager.createState2Join(endState, (Join)elementAfterJoin);
			modelManager.insertState2Join(sj);
		}else if (elementAfterJoin instanceof State){
			State2State ess=cltsFactoryManager.createState2State(endState, (State)elementAfterJoin);
			modelManager.insertState2State(ess);
			
		}  
		
	}
	
	/**
	 * 
	 * @param tracksRepresentation representation of all track from fork
	 * @param join join
	 * @param endState end state after interleaving
	 */
	private void editModelWithoutRemoveJoin ( List<RepresentationTrackFork> tracksRepresentation, Join join, State endState){
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
				
		for (RepresentationTrackFork trackRepresentation : tracksRepresentation) {
			//remove State2JoinTransition
			modelManager.removeState2Join(modelManager.getState2JoinTransition( trackRepresentation.getEndState(), join));
		}
		// connected end state to join
		State2Join sj = cltsFactoryManager.createState2Join(endState, join);
		modelManager.insertState2Join(sj);
	}
	
	/**
	 * 
	 * @param tracksRepresentation representation of all track from fork
	 * @param endState end state after interleaving
	 */
	private void editModelWithoutRemoveFinalState( List<RepresentationTrackFork> tracksRepresentation, State endState) {
		//create clts factory manager
		ELTSManager cltsFactoryManager= new ELTSManagerImpl();
				
		for (RepresentationTrackFork trackRepresentation : tracksRepresentation) {
			// remove State2FinalState
			modelManager.removeState2FinalState(modelManager.getState2Final( trackRepresentation.getEndState()));
		}
		// connected end state to final state
		State2FinalState sf = cltsFactoryManager.createState2FinalState(endState, modelManager.getFinalState());
		modelManager.insertState2FinalState(sf);
	}
	
	/**
	 * 
	 * @param tracksRepresentation representation of all track from fork
	 */
	private void editModelRemoveElements ( List<RepresentationTrackFork> tracksRepresentation){
		for (RepresentationTrackFork trackRepresentation : tracksRepresentation) {
			
			//remove all state 2 state transitions (labeled, epsilon)
			for (Transition state2StateTransition : trackRepresentation.getState2StateTransitions()) {
				if (state2StateTransition instanceof LabeledState2State)
					modelManager.removeLabeledState2State((LabeledState2State)state2StateTransition);
				else if (state2StateTransition instanceof State2State)
					modelManager.removeState2State((State2State)state2StateTransition);
			}
			//remove all states
			modelManager.removeStates(trackRepresentation.getStates());
			
			
		}
	}

	/**
	 * 
	 * @param representationAfterInterleaving representation interleaving
	 */
	private void editModelAddElements (  RepresentationTrackFork representationAfterInterleaving ){
		//add all new state
		modelManager.insertStates(representationAfterInterleaving.getStates());
		//add all new state 2 state transitions (labeled, epsilon)
		for (Transition state2StateTransition : representationAfterInterleaving.getState2StateTransitions()) {
			if (state2StateTransition instanceof LabeledState2State)
				modelManager.insertLabeledTransition((LabeledState2State)state2StateTransition);
			else if (state2StateTransition instanceof State2State)
				modelManager.insertEpsilonTransition((State2State)state2StateTransition);
		}
	}






}
