package it.univaq.di.choreos.elts.strongsimulation;

import java.io.File;
import java.io.IOException;

import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.ELTS.impl.ELTSPackageImpl;
import it.univaq.di.choreos.elts.strongsimulation.ELTSModel;
import it.univaq.di.choreos.elts.strongsimulation.Result;
import it.univaq.di.choreos.elts.strongsimulation.StrongSimulation;
import it.univaq.di.choreos.elts.strongsimulation.impl.DummyResult;
import it.univaq.di.choreos.elts.strongsimulation.impl.ELTSModelImpl;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.junit.*;

import org.junit.Assert;
/**
 * 
 * @author Alexander Perucci
 *
 */
public class StrongSimulationTest {

	private Result result;
	private String resourcesRoot="." + File.separator + "src" + File.separator + "test" + File.separator + "resources";
/**
 * 
 * @param fileNameModel file name of model
 * @return the model object
 * @throws IOException
 */
	public Model loadModel(String fileNameModel) throws IOException {
		// get fileURI to Models
		URI fileURIModel = URI.createFileURI(fileNameModel);

		// create resource from the Models
		Resource resourceModel = new XMIResourceFactoryImpl().createResource(fileURIModel);
		resourceModel.load(null);

		if (resourceModel.getContents().get(0) instanceof Model) {
			// get model
			return (Model) resourceModel.getContents().get(0);
		}

		return null;

	}

	@Before
	public void setUp() {
		ELTSPackageImpl.init();
		result = new DummyResult();
	}
	
	/**
	 * runs the test number 01, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test01() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_01"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_01"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertFalse(strongSimulation.checkStrongSimulation());
	
	}
	
	/**
	 * runs the test number 02, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test02() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_02"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_02"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertFalse(strongSimulation.checkStrongSimulation());
	
		
	}
	
	/**
	 * runs the test number 03, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test03() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_03"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_03"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	/**
	 * runs the test number 04, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test04() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_04"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_04"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}

	/**
	 * runs the test number 05, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test05() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_05"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_05"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
	
	}

	/**
	 * runs the test number 06, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test06() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_06"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_06"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}

	/**
	 * runs the test number 07, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test07() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_07"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_07"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	/**
	 * runs the test number 08, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test08() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_08"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_08"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}

	/**
	 * runs the test number 09, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test09() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_09"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_09"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}

	/**
	 * runs the test number 10, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test10() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_10"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_10"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}

	/**
	 * runs the test number 11, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test11() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_11"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_11"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}

	/**
	 * runs the test number 12, it checks the value returned by the function StrongSimulation and by the object Result
	 * @throws IOException
	 */
	@Test
	public void test12() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_12"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_12"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	@Test
	public void test13() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_13"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_13"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertFalse(strongSimulation.checkStrongSimulation());
		
	}
	
	@Test
	public void test14() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_14"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_14"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	@Test
	public void test15() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_15"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_15"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	
	@Test
	public void test16() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_16"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_16"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(secondModelManager, firstModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	@Test
	public void test17() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_17"+File.separator+"firstModel.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_17"+File.separator+"secondModel.elts";
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(secondModelManager, firstModelManager,result);
		Assert.assertTrue(strongSimulation.checkStrongSimulation());
		
	}
	
	
	@Test
	public void test18() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_18"+File.separator+"Airport_speaker_actuators_aggregator.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_18"+File.separator+"Simulate_Airport_speaker_actuators_aggregator.elts";
		String thirdFileModel = resourcesRoot+File.separator+"test_18"+File.separator+"NotSimulate_Airport_speaker_actuators_aggregator.elts";
		
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		Model thirdModel = loadModel(thirdFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		ELTSModel thirdModelManager = new ELTSModelImpl(thirdModel);
		StrongSimulation strongSimulationOne = new StrongSimulation(firstModelManager, secondModelManager,result);
		StrongSimulation strongSimulationTwo = new StrongSimulation(firstModelManager, thirdModelManager,result);
		Assert.assertTrue(strongSimulationOne.checkStrongSimulation());
		Assert.assertFalse(strongSimulationTwo.checkStrongSimulation());
		
	}
	
	
	@Test
	public void test19() throws IOException {

		String firstFileModel = resourcesRoot+File.separator+"test_19"+File.separator+"modelSezaActivityEdEpsilon.elts";
		String secondFileModel = resourcesRoot+File.separator+"test_19"+File.separator+"Simulate_ModelSezaActivityEdEpsilon.elts";
		String thirdFileModel = resourcesRoot+File.separator+"test_19"+File.separator+"NotSimulate_ModelSezaActivityEdEpsilon.elts";
		
		Model firstModel = loadModel(firstFileModel);
		Model secondModel = loadModel(secondFileModel);
		Model thirdModel = loadModel(thirdFileModel);
		ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		ELTSModel thirdModelManager = new ELTSModelImpl(thirdModel);
		StrongSimulation strongSimulationOne = new StrongSimulation(firstModelManager, secondModelManager,result);
		StrongSimulation strongSimulationTwo = new StrongSimulation(firstModelManager, thirdModelManager,result);
		Assert.assertTrue(strongSimulationOne.checkStrongSimulation());
		/*In firstFileModel c'e' un fork che fa 3 volte pushUpInfo invece in thirdFileModel lo stesso fork fa 2 volte PushUp info durante => dopo discussione
		 * abbiamo detto che e' corretto.
		*/
		Assert.assertTrue(strongSimulationTwo.checkStrongSimulation());
		
	}
	
}
