package it.univaq.di.choreos.coordderivator;

import it.univaq.di.choreos.CLTS.ActivityLoop;
import it.univaq.di.choreos.CLTS.ActivityLoop2State;
import it.univaq.di.choreos.CLTS.FinalState;
import it.univaq.di.choreos.CLTS.LabeledTransition;
import it.univaq.di.choreos.CLTS.Model;
import it.univaq.di.choreos.CLTS.NamedElement;
import it.univaq.di.choreos.CLTS.State2Join;
import it.univaq.di.choreos.CLTS.Transition;
import it.univaq.di.choreos.coordderivator.util.CoordUtil;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Condition;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Tuple;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.util.EcoreUtil;

public class CoordDerivator {
	private List<ActivityLoop> activeLoops;
	private List<NamedElement> activeElements;
	private List<NamedElement> joinUnlockeds;
	private NamedElementCoordMap activeElementsMapCoord;
	private NamedElementCoordMap boundaryJoin;
	private List<TupleStored> tupleStoreds;
	private List<TupleStored> hangingStateFork;
	private List<Coord> coordsModel;
	private boolean isSetStartState;
	private CltsWrapper cltsWapper;


	public CoordDerivator(Model model) {
		activeElements = new ArrayList<NamedElement>();
		activeLoops = new ArrayList<ActivityLoop>();
		joinUnlockeds = new ArrayList<NamedElement>();
		activeElementsMapCoord = new NamedElementCoordMap();
		boundaryJoin = new NamedElementCoordMap();
		tupleStoreds = new ArrayList<TupleStored>();
		hangingStateFork = new ArrayList<TupleStored>();
		coordsModel = new ArrayList<Coord>();
		cltsWapper = new CltsWrapper(model);

		isSetStartState = false;
	}

	public void init() {
		List<LabeledTransition> startLabeledTransitions = cltsWapper.getStartLabeledTransitions();

		String coordName = cltsWapper.getCoordName(startLabeledTransitions.get(0));

		List<Transition> transitionAdd = new ArrayList<Transition>();
		transitionAdd.add(startLabeledTransitions.get(0));
		insertTuple(coordName, transitionAdd);

	}

	private void insertTuple(String coordName, List<Transition> transitions) {
		for (Transition transition : transitions) {
			Coord coordModel;

			// SET coordModel to insert tuple
			if (transition instanceof LabeledTransition) {
				coordModel = CoordUtil.createCoordModel(coordsModel, cltsWapper.getCoordName(transition));
			} else {
				coordModel = CoordUtil.createCoordModel(coordsModel, coordName);
			}

			Tuple tuple = CoordUtil.createTuple();

			NamedElement targetNamedElement = cltsWapper.getStateTarget(transition);
			NamedElement sourceNamedElement = cltsWapper.getStateSource(transition);
			State sourceState = CoordUtil.createState(coordModel, sourceNamedElement.getName());
			State targetState = CoordUtil.createState(coordModel, targetNamedElement.getName());
			Condition condition = CoordUtil.createCondition(transition.getCondition());

			TupleStored ts = new TupleStored(sourceState.getName(), targetState.getName(), condition.getExpression(), coordModel.getName());
			if (tupleStoreds.contains(ts)) {
				break;
			}

			tupleStoreds.add(ts);

			// SET Base information
			if (!isSetStartState) {
				coordModel.setInitialState(sourceState);
				isSetStartState = true;
			}
			tuple.setSourceState(sourceState);
			tuple.setTargetState(targetState);
			tuple.setCondition(condition);
			if (transition instanceof LabeledTransition) {
				String operationName = ((LabeledTransition) transition).getLabel().getName();
				tuple.setAllowedOperation(CoordUtil.createAllowedOperation(coordModel, operationName));
			}

			// SET allowedComponentInTargetState
			List<Transition> nextTransitions = cltsWapper.getNextTransitions(targetNamedElement);
			if (nextTransitions.size() == 1) {
				if (nextTransitions.get(0) instanceof LabeledTransition) {
					tuple.getAllowedComponentInTargetState().add(CoordUtil.createParticipant(coordModel, cltsWapper.getCoordName(nextTransitions.get(0))));
				}
			}

			// SET notify and wait
			if (transition instanceof State2Join) {
				// setNotify
				List<String> coordsNotify = new ArrayList<String>();
				for (NamedElement boundary : cltsWapper.getStateBoundaryJoin(targetNamedElement)) {
					if (!boundary.getName().equals(sourceNamedElement.getName())) {
						List<String> coordsBoudaryName = boundaryJoin.get(boundary);
						for (String coordBoudaryName : coordsBoudaryName) {
							if (!coordsNotify.contains(coordBoudaryName)) {
								coordsNotify.add(coordBoudaryName);
							}
						}
					}
				}
				tuple.setNotify(CoordUtil.createNotify(coordModel, sourceState, coordsNotify));

				// set wait
				NamedElementCoordMap stateWaits = new NamedElementCoordMap();
				for (NamedElement boundary : cltsWapper.getStateBoundaryJoin(targetNamedElement)) {
					if (!boundary.getName().equals(sourceNamedElement.getName())) {
						stateWaits.put(boundary, boundaryJoin.get(boundary));
					}
				}
				tuple.setWait(CoordUtil.createWait(coordModel, stateWaits));

			}

			// check set targetNamedElement in boundaryJoin
			List<Transition> netxTransitionsTargetNamedElement = cltsWapper.getNextTransitions(targetNamedElement);
			if (netxTransitionsTargetNamedElement.size() > 0) {
				if (netxTransitionsTargetNamedElement.get(0) instanceof State2Join) {
					boundaryJoin.put(targetNamedElement, coordModel.getName());
				}
			}

			activeElements.add(targetNamedElement);
			activeElementsMapCoord.put(targetNamedElement, coordModel.getName());
			coordModel.getTuples().add(tuple);
		}

		insertNextTransition();
	}

	private void insertNextTransition() {
		if (activeElements.isEmpty()) {
			return;
		}
		NamedElement activeElement = removeNamedElement();

		if (activeElement == null) {
			activeElement = selectNamedElementToContinue();
		}

		String coordName = activeElementsMapCoord.remove(activeElement);

		if (activeElement instanceof ActivityLoop) {
			if (activeLoops.contains((activeElement))) {
				// activeLoops.remove((ActivityLoop) activeElement); // Not Remove
				Transition tempLoopInTransition = cltsWapper.getTransitionLoopIn((ActivityLoop) activeElement);
				Transition tempLoopOutTransition = cltsWapper.getTransitionLoopOut((ActivityLoop) activeElement);

				Transition loopInTransition = null;
				Transition loopOutTransition = null;

				if (tempLoopInTransition.getCondition() != null && !tempLoopInTransition.getCondition().isEmpty()) {
					// get condition
					loopInTransition = tempLoopInTransition;
					loopOutTransition = EcoreUtil.copy(tempLoopOutTransition);
					loopOutTransition.setCondition("!(" + tempLoopInTransition.getCondition() + ")");
				} else {
					// get number loop
					loopInTransition = EcoreUtil.copy(tempLoopInTransition);
					loopInTransition.setCondition("_count() > " + ((ActivityLoop2State) loopInTransition).getCardinality());
					loopOutTransition = EcoreUtil.copy(tempLoopOutTransition);
					loopOutTransition.setCondition("!(" + loopInTransition.getCondition() + ")");
				}

				List<Transition> transitionAdd = new ArrayList<Transition>();
				transitionAdd.add(loopInTransition);
				transitionAdd.add(loopOutTransition);

				insertTuple(coordName, transitionAdd);

			} else {
				activeLoops.add((ActivityLoop) activeElement);
				Transition loopInTransition = cltsWapper.getTransitionLoopIn((ActivityLoop) activeElement);
				// remove condition
				Transition temp = EcoreUtil.copy(loopInTransition);
				temp.setCondition(null);
				List<Transition> transitionAdd = new ArrayList<Transition>();
				transitionAdd.add(temp);
				insertTuple(coordName, transitionAdd);
			}

		} else {

			//check if nextTransition is state2Join
			if (!(activeElement instanceof FinalState)) {
				Transition transition = cltsWapper.getNextTransitions(activeElement).get(0);
				if (transition instanceof State2Join) {
					TupleStored st = new TupleStored(activeElement.getName(), ((State2Join) transition).getTarget().getName(), "", coordName);
					hangingStateFork.add(st);
				}

			}
			insertTuple(coordName, cltsWapper.getNextTransitions(activeElement));
		}

	}

	private NamedElement removeNamedElement() {
		for (int i = 0; i < activeElements.size(); i++) {
			if (checkNamedElementRemove(activeElements.get(i)))
				return activeElements.remove(i);
		}
		return null;
	}

	private boolean checkNamedElementRemove(NamedElement element) {
		for (Transition transition : cltsWapper.getNextTransitions(element)) {
			if (transition instanceof State2Join) {
				return false;
			}
		}
		return true;
	}

	private NamedElement selectNamedElementToContinue() {
		for (int i = 0; i < activeElements.size(); i++) {
			NamedElement join = null;
			List<Transition> nextTransitionsElement = cltsWapper.getNextTransitions(activeElements.get(i));
			if (nextTransitionsElement.size() > 0 && nextTransitionsElement.get(0) instanceof State2Join) {
				join = ((State2Join) cltsWapper.getNextTransitions(activeElements.get(i)).get(0)).getTarget();
				if (joinUnlockeds.contains(join))
					return activeElements.remove(i);
			}
		}

		
		//if you are here it means that the target of the join transitions in the list, not 'unlocked then I have to unlock one
		for (int i = 0; i < activeElements.size(); i++) {
			List<Transition> nextTransitionsElement = cltsWapper.getNextTransitions(activeElements.get(i));
			if (nextTransitionsElement.size() > 0 && nextTransitionsElement.get(0) instanceof State2Join) {
				List<NamedElement> boundaries = cltsWapper.getStateBoundaryJoin(((State2Join) cltsWapper.getNextTransitions(activeElements.get(i)).get(0)).getTarget());
				//verify that the list contains all the boudary, if true remove this item
				boolean remove = true;
				for (NamedElement boundary : boundaries) {
					if (!boundaryJoin.containsKey(boundary)) {
						remove = false;
						break;
					}
				}
				if (remove) {
					joinUnlockeds.add(((State2Join) cltsWapper.getNextTransitions(activeElements.get(i)).get(0)).getTarget());
					return activeElements.remove(i);
				}

			}

		}
		return null;
	}

	public List<Coord> getCoords() {
		return coordsModel;
	}


}
