package it.univaq.di.choreos.coordderivator.util;

import it.univaq.di.choreos.CLTS.NamedElement;
import it.univaq.di.choreos.coordderivator.NamedElementCoordMap;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Condition;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Notify;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.NotifyElement;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Tuple;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Wait;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.WaitElement;

import java.util.List;

public class CoordUtil {

	public static Coord createCoordModel(List<Coord> coordsModel, String name) {
		for (Coord model : coordsModel) {
			if (model.getName().equals(name))
				return model;
		}
		Coord model = CoordmetamodelFactory.eINSTANCE.createCoord();
		model.setName(name);
		coordsModel.add(model);
		return model;
	}

	public static State createState(Coord coordModel, String name) {
		for (State state : coordModel.getStates()) {
			if (state.getName().equals(name))
				return state;
		}
		State state = CoordmetamodelFactory.eINSTANCE.createState();
		state.setName(name);
		coordModel.getStates().add(state);
		return state;
	}

	public static Participant createParticipant(Coord coordModel, String name) {
		for (Participant participant : coordModel.getParticipants()) {
			if (participant.getName().equals(name))
				return participant;
		}
		Participant participant = CoordmetamodelFactory.eINSTANCE.createParticipant();
		participant.setName(name);
		coordModel.getParticipants().add(participant);
		return participant;
	}

	public static AllowedOperation createAllowedOperation(Coord coordModel, String name) {
		for (AllowedOperation allowedOperation : coordModel.getOperations()) {
			if (allowedOperation.getName().equals(name))
				return allowedOperation;
		}
		AllowedOperation allowedOperation = CoordmetamodelFactory.eINSTANCE.createAllowedOperation();
		allowedOperation.setName(name);
		coordModel.getOperations().add(allowedOperation);
		return allowedOperation;
	}
	
	public static Condition createCondition(String name) {
		Condition condition = CoordmetamodelFactory.eINSTANCE.createCondition();
		if (name == null || name.isEmpty()) {
			condition.setExpression("true");
		} else{
			condition.setExpression(name);	
		}
		return condition;
	}

	public static Tuple createTuple() {
		return CoordmetamodelFactory.eINSTANCE.createTuple();
	}
	
	public static Notify createNotify(Coord coordModel, State stateNotify, List<String> participants){
		Notify notify = CoordmetamodelFactory.eINSTANCE.createNotify();
		NotifyElement notifyElement = CoordmetamodelFactory.eINSTANCE.createNotifyElement();
		notifyElement.setState(stateNotify);
		
		for (String participantName : participants) {
			Participant participant = createParticipant(coordModel, participantName);
			notifyElement.getParticipants().add(participant);
		}
		
		notify.getElements().add(notifyElement);
		return notify;
	}

	public static Wait createWait(Coord coordModel, NamedElementCoordMap waits){
		Wait wait = CoordmetamodelFactory.eINSTANCE.createWait();
		for (NamedElement namedElementWait : waits.keySet()) {
			State stateWait = createState(coordModel, namedElementWait.getName());
			WaitElement waitElement = CoordmetamodelFactory.eINSTANCE.createWaitElement();
			waitElement.setState(stateWait);
			for (String participantName : waits.get(namedElementWait)) {
				Participant participant = createParticipant(coordModel, participantName);
				waitElement.getParticipants().add(participant);
			}
			wait.getElements().add(waitElement);
		}
		return wait;
	}
	

}
