package it.univaq.di.choreos.coordderivator.util;

import java.util.List;


import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Condition;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Notify;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.NotifyElement;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.State;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Tuple;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Wait;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.WaitElement;

public class CoordUtilTest {

	public static boolean isEquals(List<Coord> expected, List<Coord> result) {
		if (expected.size() != result.size())
			return false;

		for (Coord modelExpected : expected) {

			Coord modelResult = null;
			for (Coord model : result) {
				if (modelExpected.getName().equals(model.getName())) {
					modelResult = model;
					break;
				}
			}

			if (modelResult == null)
				return false;
			
			if (!isEquals(modelExpected, modelResult))
				return false;
		}

		return true;
	}

	private static boolean isEquals(Coord expected, Coord result) {
		if (!isEqualsInitialState(expected.getInitialState(), result.getInitialState()))
			return false;
		if (!isEqualsOperations(expected.getOperations(), result.getOperations()))
			return false;
		if (!isEqualsParticipants(expected.getParticipants(), result.getParticipants()))
			return false;
		if (!isEqualsStates(expected.getStates(), result.getStates()))
			return false;
		if (!isEqualsTuples(expected.getTuples(), result.getTuples()))
			return false;

		return true;
	}
	
	private static boolean isEqualsInitialState(State expectedInitialState, State resultInitialState){
		if (expectedInitialState == null && resultInitialState != null)
			return false;
		else if (expectedInitialState != null && resultInitialState == null)
			return false;
		else if (expectedInitialState != null && resultInitialState != null && !expectedInitialState.getName().equals(resultInitialState.getName()))
			return false;
		
		return true;
	}
	
	private static boolean isEqualsOperations(List<AllowedOperation> expecteds, List<AllowedOperation> results) {
		if (expecteds.size() != results.size())
			return false;

		for (AllowedOperation allowedOperationExpected : expecteds) {
			AllowedOperation allowedOperationResult = null;
			for (AllowedOperation allowedOperation : results) {
				if (allowedOperationExpected.getName().equals(allowedOperation.getName())) {
					allowedOperationResult = allowedOperation;
					break;
				}
			}
			if (allowedOperationResult == null)
				return false;
		}

		return true;
	}

	private static boolean isEqualsParticipants(List<Participant> expecteds, List<Participant> results) {
		if (expecteds.size() != results.size())
			return false;

		for (Participant participantExpected : expecteds) {
			Participant participantResult = null;
			for (Participant participant : results) {
				if (participantExpected.getName().equals(participant.getName())) {
					participantResult = participant;
					break;
				}
			}
			if (participantResult == null)
				return false;
		}

		return true;
	}

	private static boolean isEqualsStates(List<State> expecteds, List<State> results) {
		if (expecteds.size() != results.size())
			return false;

		for (State stateExpected : expecteds) {
			State stateResult = null;
			for (State state : results) {
				if (stateExpected.getName().equals(state.getName())) {
					stateResult = state;
					break;
				}
			}
			if (stateResult == null)
				return false;
		}

		return true;
	}

	private static boolean isEqualsTuples(List<Tuple> expecteds, List<Tuple> results) {
		if (expecteds.size() != results.size())
			return false;
		for (Tuple tupleExpected : expecteds) {
			Tuple correctTuple = null;
			for (Tuple tupleResult : results) {
				if (checkTupleAllowedOperation(tupleExpected.getAllowedOperation(), tupleResult.getAllowedOperation()) &&
						checkTupleCondition(tupleExpected.getCondition(), tupleResult.getCondition()) &&
						checkTupleState(tupleExpected.getSourceState(), tupleResult.getSourceState()) &&
						checkTupleState(tupleExpected.getTargetState(), tupleResult.getTargetState()) &&
						checkTupleAllowedComponentInTargetState(tupleExpected.getAllowedComponentInTargetState(), tupleResult.getAllowedComponentInTargetState()) &&
						checkTupleNotify(tupleExpected.getNotify(),tupleResult.getNotify())&&
						checkTupleWait(tupleExpected.getWait(), tupleResult.getWait())) {
					correctTuple = tupleResult;
					break;
				}
			}
			if (correctTuple == null) {
				return false;
			}
		}

		return true;
	}

	private static boolean checkTupleAllowedOperation(AllowedOperation expectedOperation, AllowedOperation resultOperation) {
		if (expectedOperation == null) {
			if (resultOperation != null)
				return false;
		} else {
			if (resultOperation == null)
				return false;
			else {
				if (!expectedOperation.getName().equals(resultOperation.getName()))
					return false;
			}
		}

		return true;
	}

	private static boolean checkTupleCondition(Condition expectedCondition, Condition resultCondition) {
		if (expectedCondition == null) {
			if (resultCondition != null)
				return false;
		} else {
			if (resultCondition == null)
				return false;
			else {
				if (!expectedCondition.getExpression().equals(resultCondition.getExpression()))
					return false;
			}
		}
		return true;
	}

	private static boolean checkTupleState(State expectedState, State resultState) {
		if (expectedState == null) {
			if (resultState != null)
				return false;
		} else {
			if (resultState == null)
				return false;
			else {
				if (!expectedState.getName().equals(resultState.getName()))
					return false;
			}
		}
		return true;
	}

	private static boolean checkTupleAllowedComponentInTargetState (List<Participant> expectedParticipants, List<Participant> resultParticipants){
		if (expectedParticipants.size() != resultParticipants.size())
			return false;
		
		for (Participant expectedParticipant : expectedParticipants) {
			Participant correctParticipant = null;
			for (Participant resultParticipant : resultParticipants) {
				if (checkTupleParticipant(expectedParticipant, resultParticipant)){
					correctParticipant = resultParticipant;
					break;
				}
			}
			if (correctParticipant == null) {
				return false;
			}
		}
		return true;
	}
	
	private static boolean checkTupleParticipant ( Participant expectedParticipant, Participant resultParticipant){
		if (expectedParticipant == null) {
			if (resultParticipant != null)
				return false;
		} else {
			if (resultParticipant == null)
				return false;
			else {
				if (!expectedParticipant.getName().equals(resultParticipant.getName()))
					return false;
			}
		}
		return true;
	}
	
	private static boolean checkTupleNotify(Notify expectdNotify, Notify resultNotify){
		if (expectdNotify == null){
			if (resultNotify != null)
				return false;
		}else{
			if (resultNotify == null)
				return false;
			else {
				if (expectdNotify.getElements().size()!=resultNotify.getElements().size())
					return false;
				
				for (NotifyElement expectedNotifyElement : expectdNotify.getElements()) {
					boolean found = false;
					for (NotifyElement resultNotifyElement : resultNotify.getElements()) {
						if (expectedNotifyElement.getState().getName().equals(resultNotifyElement.getState().getName())){
							if (expectedNotifyElement.getParticipants().size() != resultNotifyElement.getParticipants().size())
								return false;
							boolean foundParticipant = false;
							for (Participant expectedParticipant : expectedNotifyElement.getParticipants()) {
								for (Participant resultParticipant : resultNotifyElement.getParticipants()) {
									if (expectedParticipant.getName().equals(resultParticipant.getName())){
										foundParticipant = true;
										break;
									}
								}
								if (!foundParticipant){
									return false;
								}
							}
							found = true;
							break;
						}
					}
					if (!found){
						return false;
					}
				}
			}
		}
		
		return true;
	}
	private static boolean checkTupleWait(Wait expectdWait, Wait resultWait){
		if (expectdWait == null){
			if (resultWait != null)
				return false;
		}else{
			if (resultWait == null)
				return false;
			else {
				if (expectdWait.getElements().size()!=resultWait.getElements().size())
					return false;
				
				for (WaitElement expectedWaitElement : expectdWait.getElements()) {
					boolean found = false;
					for (WaitElement resultWaitElement : resultWait.getElements()) {
						if (expectedWaitElement.getState().getName().equals(resultWaitElement.getState().getName())){
							if (expectedWaitElement.getParticipants().size() != resultWaitElement.getParticipants().size())
								return false;
							boolean foundParticipant = false;
							for (Participant expectedParticipant : expectedWaitElement.getParticipants()) {
								for (Participant resultParticipant : resultWaitElement.getParticipants()) {
									if (expectedParticipant.getName().equals(resultParticipant.getName())){
										foundParticipant = true;
										break;
									}
								}
								if (!foundParticipant){
									return false;
								}
							}
							found = true;
							break;
						}
					}
					if (!found){
						return false;
					}
				}
			}
		}
		return true;
	}


}
