package it.univaq.di.choreos.projectmodel;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.m2m.atl.common.ATLExecutionException;
import org.eclipse.m2m.atl.core.ATLCoreException;
import org.eclipse.m2m.atl.core.IExtractor;
import org.eclipse.m2m.atl.core.IInjector;
import org.eclipse.m2m.atl.core.IModel;
import org.eclipse.m2m.atl.core.IReferenceModel;
import org.eclipse.m2m.atl.core.ModelFactory;
import org.eclipse.m2m.atl.core.emf.EMFExtractor;
import org.eclipse.m2m.atl.core.emf.EMFInjector;
import org.eclipse.m2m.atl.core.emf.EMFModelFactory;
import org.eclipse.m2m.atl.core.launch.ILauncher;
import org.eclipse.m2m.atl.engine.emfvm.launch.EMFVMLauncher;


public class ProjectCltsModel {
	private static final String RESOURCE_PATH = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator;
	private static final String MODULES = RESOURCE_PATH+"clts2elts.atl";
	private static final String METAMODEL_IN = RESOURCE_PATH+"CLTS.ecore";
	private static final String METAMODEL_OUT = RESOURCE_PATH+"ELTS.ecore";
	private static final String LIBRARY_CONFIG = RESOURCE_PATH+"Config.asm";
	
	
	private Map<String, Object> options;
	private IModel inModel;
	private IModel outModel;

	
	public ProjectCltsModel() throws IOException {
		options = new HashMap<String, Object>();
		options.put("supportUML2Stereotypes", "false");
		options.put("printExecutionTime", "false");
		options.put("OPTION_CONTENT_TYPE", "false");
		options.put("allowInterModelReferences", "false");
		options.put("step", "false");
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("ecore", new EcoreResourceFactoryImpl());
	}

	
	public String projects(String cltsModel, String roleName) {
		try {
			writeConfigFile(roleName);
			loadModels(cltsModel);
			doClts2elts(new NullProgressMonitor());
			String tempfolder = createTempFolder();
			File eltsTemp = new File(tempfolder+File.separator+"temp.elts");
			saveModels(eltsTemp.getAbsolutePath());
			String eltsContent = FileUtils.readFileToString(eltsTemp);
			FileUtils.deleteQuietly(new File(tempfolder));
			deleteConfigFile();
			return eltsContent;
		} catch (ATLCoreException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ATLExecutionException e) {
			e.printStackTrace();
		}
		
		return "";
	}

	
	private void loadModels(String contentModel) throws ATLCoreException {
		try {
			ModelFactory factory = new EMFModelFactory();
			IInjector injector = new EMFInjector();
			IReferenceModel inmodelMetamodel = factory.newReferenceModel();
			injector.inject(inmodelMetamodel, METAMODEL_IN);
			IReferenceModel outmodelMetamodel = factory.newReferenceModel();
			injector.inject(outmodelMetamodel, METAMODEL_OUT);
			this.inModel = factory.newModel(inmodelMetamodel);
			String tempfolder = createTempFolder();
			File tempClts = new File(tempfolder+File.separator + "temp.clts");
			FileUtils.writeStringToFile(tempClts, contentModel);
			injector.inject(inModel, tempClts.getAbsolutePath());
			this.outModel = factory.newModel(outmodelMetamodel);
			FileUtils.deleteQuietly(new File(tempfolder));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	
	private void saveModels(String outModelPath) throws ATLCoreException {
		IExtractor extractor = new EMFExtractor();
		extractor.extract(outModel, outModelPath);
	}

	
	private Object doClts2elts(IProgressMonitor monitor) throws ATLCoreException, IOException, ATLExecutionException {
		ILauncher launcher = new EMFVMLauncher();
		List<InputStream> inputStreamsToClose = new ArrayList<InputStream>();
		Map<String, Object> launcherOptions = getOptions();
		InputStream launcherLibrary = new FileInputStream(LIBRARY_CONFIG);
		launcher.initialize(launcherOptions);
		launcher.addInModel(inModel, "IN", "INMODEL");
		launcher.addOutModel(outModel, "OUT", "OUTMODEL");
		launcher.addLibrary("Config", launcherLibrary);
		InputStream[] modulesStreams = getModulesList();
		inputStreamsToClose.addAll(Arrays.asList(modulesStreams));
		Object result = launcher.launch("run", monitor, launcherOptions, (Object[]) modulesStreams);
		for (InputStream inputStream : inputStreamsToClose) {
			inputStream.close();
		}
		return result;
	}

	
	private InputStream[] getModulesList() throws IOException {
		InputStream[] modules = null;
		String[] moduleNames = MODULES.split(",");
		modules = new InputStream[moduleNames.length];
		for (int i = 0; i < moduleNames.length; i++) {
			String asmModulePath = new Path(moduleNames[i].trim()).removeFileExtension().addFileExtension("asm").toString();
			modules[i] = new FileInputStream(asmModulePath);
		}
		return modules;
	}

	
	private Map<String, Object> getOptions() {
		return options;
	}

	private void writeConfigFile(String roleName) {
		String configFileContent = "<?xml version = '1.0' encoding = 'ISO-8859-1' ?>" + "<asm version=\"1.0\" name=\"0\">" + "<cp>" + "<constant value=\"Config\"/>" + "<constant value=\"roleName\"/>" + "<constant value=\"J\"/>" + "<constant value=\"main\"/>" + "<constant value=\"A\"/>" + "<constant value=\"" + roleName + "\"/>" + "<constant value=\"2:37-2:42\"/>" + "<constant value=\"self\"/>" + "</cp>" + "<field name=\"1\" type=\"2\"/>" + "<operation name=\"3\">" + "<context type=\"4\"/>" + "<parameters>" + "</parameters>" + "<code>" + "<getasm/>" + "<push arg=\"5\"/>" + "<set arg=\"1\"/>" + "</code>" + "<linenumbertable>" + "<lne id=\"6\" begin=\"1\" end=\"1\"/>" + "</linenumbertable>" + "<localvariabletable>" + "<lve slot=\"0\" name=\"7\" begin=\"0\" end=\"2\"/>" + "</localvariabletable>" + "</operation>" + "</asm>";
		try {
			FileUtils.writeStringToFile(new File(LIBRARY_CONFIG), configFileContent);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void deleteConfigFile() {
		FileUtils.deleteQuietly(new File(LIBRARY_CONFIG));
	}


	private String createTempFolder(){
		String tempFolder = getTempFolder();
		File tempFolderDir = new File(tempFolder);
		try {
			FileUtils.forceMkdir(tempFolderDir);
			return tempFolder;
		} catch (IOException e) {
			e.printStackTrace();
		}finally {
			try {
				FileUtils.deleteDirectory(tempFolderDir);
			} catch (IOException e) {
			}
		}
		return "";
	}

	private String getTempFolder() {
		return FileUtils.getTempDirectoryPath() + File.separator + getRandomDir() + File.separator;
	}

	private String getRandomDir() {
		return new Date().getTime() + "";
	}


}
