package it.univaq.di.choreos.synthesis.behavioursimulator.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import it.univaq.di.choreos.CLTS.CLTSMetamodelParser;
import it.univaq.di.choreos.CLTS.Role;
import it.univaq.di.choreos.ELTS.ELTSMetamodelParser;
import it.univaq.di.choreos.ELTS.Model;
import it.univaq.di.choreos.elts.strongsimulation.StrongSimulation;
import it.univaq.di.choreos.elts.strongsimulation.impl.DummyResult;
import it.univaq.di.choreos.elts.strongsimulation.impl.ELTSModelImpl;
import it.univaq.di.choreos.projectmodel.ProjectCltsModel;
import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulator;
import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulatorException;
import it.univaq.di.choreos.synthesis.datamodel.CLTSModel;
import it.univaq.di.choreos.synthesis.datamodel.ELTSModel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BehaviourSimulatorImpl implements BehaviourSimulator {

	private static Logger logger = LoggerFactory.getLogger(BehaviourSimulatorImpl.class);

	public boolean simulation(ELTSModel firstELTSModel, ELTSModel secondELTSModel) throws BehaviourSimulatorException {
		logger.info("start simulation");
		Model firstModel = ELTSMetamodelParser.getEltsModel(firstELTSModel.getEltsModel());
		Model secondModel = ELTSMetamodelParser.getEltsModel(secondELTSModel.getEltsModel());
		it.univaq.di.choreos.elts.strongsimulation.ELTSModel firstModelManager = new ELTSModelImpl(firstModel);
		it.univaq.di.choreos.elts.strongsimulation.ELTSModel secondModelManager = new ELTSModelImpl(secondModel);
		StrongSimulation strongSimulation = new StrongSimulation(firstModelManager, secondModelManager, new DummyResult());
		logger.info("end simulation");
		return strongSimulation.checkStrongSimulation();
	}

	public ELTSModel projection(CLTSModel cltsModel, String participantName) throws BehaviourSimulatorException {
		logger.info("start projection");
		ProjectCltsModel projectCltsModel;
		String eltsContent = "";
		try {
			projectCltsModel = new ProjectCltsModel();
			eltsContent = projectCltsModel.projects(cltsModel.getCltsModel(), participantName);
		} catch (IOException e) {
			e.printStackTrace();
		}

		ELTSModel result = new ELTSModel(eltsContent, participantName);
		logger.info("end projection");
		return result;
	}
	
	public List<ELTSModel> projections(CLTSModel cltsModel) throws BehaviourSimulatorException {
		logger.info("start projections");
		it.univaq.di.choreos.CLTS.Model clts = CLTSMetamodelParser.getCltsModel(cltsModel.getCltsModel());
		List<ELTSModel> eltsModels = new ArrayList<ELTSModel>();
		
		for (Role role : clts.getRolesDefinition().getRoles()) {
			ELTSModel elts = projection(cltsModel, role.getName());
			eltsModels.add(elts);
		}
		logger.info("end projections");
		return eltsModels;
	}

}
