package it.univaq.di.choreos.synthesis.behavioursimulator.rest;

import java.util.List;

import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulator;
import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulatorException;
import it.univaq.di.choreos.synthesis.behavioursimulator.impl.BehaviourSimulatorImpl;
import it.univaq.di.choreos.synthesis.datamodel.CLTSModel;
import it.univaq.di.choreos.synthesis.datamodel.ELTSModel;

import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RestBehaviourSimulator {

	private static Logger logger = LoggerFactory.getLogger(RestBehaviourSimulator.class);
	private BehaviourSimulator simulator = new BehaviourSimulatorImpl();
	
	
	@Path("/simulation")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response simulation(Simulation simulation) {	
		if (simulation.getFirstELTSModel() == null || simulation.getSecondELTSModel() == null) {
			return Response.status(Status.BAD_REQUEST).build();
		}

		try {
			boolean result = simulator.simulation(simulation.getFirstELTSModel(), simulation.getSecondELTSModel());
			return Response.ok(result).build();
		} catch (BehaviourSimulatorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

	}

	@Path("/projection")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response projection(Projection projection) {
		
		if (projection.getCltsModel() == null || projection.getParticipantName()==null) {
			return Response.status(Status.BAD_REQUEST).build();
		}

		try {
			ELTSModel result = simulator.projection(projection.getCltsModel(), projection.getParticipantName());
			return Response.ok(result).build();
		} catch (BehaviourSimulatorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

	}
	
	
	@Path("/projections")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	@Produces(MediaType.APPLICATION_XML)
	public Response projections(CLTSModel cltsModel) {
		
		if (cltsModel == null || cltsModel.getCltsModel() == null ) {
			return Response.status(Status.BAD_REQUEST).build();
		}

		try {
			List<ELTSModel> projections = simulator.projections(cltsModel);
			Projections result = new Projections();
			result.setProjections(projections);
			return Response.ok(result).build();
		} catch (BehaviourSimulatorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

	}

}
