package it.univaq.di.choreos.synthesis.behavioursimulator.rest.client;


import java.util.List;

import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulator;
import it.univaq.di.choreos.synthesis.behavioursimulator.BehaviourSimulatorException;
import it.univaq.di.choreos.synthesis.behavioursimulator.rest.Projection;
import it.univaq.di.choreos.synthesis.behavioursimulator.rest.Projections;
import it.univaq.di.choreos.synthesis.behavioursimulator.rest.Simulation;
import it.univaq.di.choreos.synthesis.datamodel.CLTSModel;
import it.univaq.di.choreos.synthesis.datamodel.ELTSModel;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.apache.cxf.jaxrs.client.WebClient;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;

public class BehaviourSimulatorClient implements BehaviourSimulator {

	private String host;

	public BehaviourSimulatorClient(String host) {
		this.host = host;
	}


	private WebClient setupClient() {
		WebClient client = WebClient.create(host);

		// remove time out
		// not proud of it!
		HTTPConduit http = (HTTPConduit) WebClient.getConfig(client).getConduit();
		HTTPClientPolicy httpClientPolicy = new HTTPClientPolicy();
		httpClientPolicy.setConnectionTimeout(0);// indefined
		httpClientPolicy.setReceiveTimeout(0);// indefined
		http.setClient(httpClientPolicy);

		return client;
	}

	public boolean simulation(ELTSModel firstELTSModel, ELTSModel secondELTSModel) throws BehaviourSimulatorException {
		WebClient client = setupClient();
		client.path("simulation");

		try {
			Response response = client.post(new Simulation(firstELTSModel, secondELTSModel));
			if (Status.BAD_REQUEST.equals(response.getStatus()) || Status.INTERNAL_SERVER_ERROR.equals(response.getStatus())) {
				return false;
			}
			return true;
		} catch (WebApplicationException e) {
			return false;
		}
	}

	public ELTSModel projection(CLTSModel cltsModel, String participantName) throws BehaviourSimulatorException {
		WebClient client = setupClient();
		client.path("projection");

		try {
			return client.post(new Projection(cltsModel, participantName), ELTSModel.class);

		} catch (WebApplicationException e) {
			throw new BehaviourSimulatorException();
		}
	}
	
	public List<ELTSModel> projections(CLTSModel cltsModel) throws BehaviourSimulatorException {
		WebClient client = setupClient();
		client.path("projections");

		try {
			Projections projections = client.post(cltsModel, Projections.class);
			return projections.getProjections();

		} catch (WebApplicationException e) {
			throw new BehaviourSimulatorException();
		}
	}
}
