package it.univaq.di.choreos.synthesis.cdgenerator.impl;

import it.univaq.di.choreos.synthesis.cdgenerator.CDGenerator;
import it.univaq.di.choreos.synthesis.cdgenerator.CDGeneratorException;
import it.univaq.di.choreos.synthesis.cdgenerator.impl.easyesb.CDService;
import it.univaq.di.choreos.synthesis.cdgenerator.impl.easyesb.EasyESBCDGenerator;
import it.univaq.di.choreos.synthesis.cdgenerator.utility.CDGeneratorUtility;
import it.univaq.di.choreos.synthesis.datamodel.CD;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CDGeneratorImpl implements CDGenerator {

	private static final String TARGZ_SUFFIX = ".tar.gz";
	
	private EasyESBCDGenerator easyESBCDGenerator = new EasyESBCDGenerator();
	private String cdHomeRepository; 
	private static Logger logger = LoggerFactory.getLogger(CDGeneratorImpl.class);
	
	public CDGeneratorImpl(String cdHomeRepository) {
		this.cdHomeRepository = cdHomeRepository;
	}

	public String generate(CD cd) throws CDGeneratorException {
		try {
			logger.info("start generating");
			CDService  cdService = new CDService(cd);
			String baseCDDir="";
			if (cd.getParentFolderName()==null){
				baseCDDir = CDGeneratorUtility.getRandomDir();
			}else{
				baseCDDir = cd.getParentFolderName();
			}
			
			String cdOutputDir = null;
			if (cd.getChoreographyName()!=null && !cd.getChoreographyName().equals("")) {
				cdOutputDir = cdHomeRepository + File.separator + cd.getChoreographyName() + File.separator + baseCDDir;
				FileUtils.forceMkdir(new File(cdHomeRepository + File.separator + cd.getChoreographyName()));
			} else {
				cdOutputDir = cdHomeRepository + File.separator + baseCDDir;
			}
			String cdfileName = cdOutputDir + File.separator + cdService.getCdName() + TARGZ_SUFFIX;
			FileUtils.forceMkdir(new File(cdOutputDir));
			easyESBCDGenerator.generate( cdService, cdfileName );
			String result = null;
			if (cd.getChoreographyName()!=null && !cd.getChoreographyName().equals("")) {
				result = "/" + cd.getChoreographyName() + "/"+ baseCDDir + "/" + cdService.getCdName() + TARGZ_SUFFIX;
			} else {
				result = "/" + baseCDDir + "/" + cdService.getCdName() + TARGZ_SUFFIX;
			}
			 
			logger.info("end generating cd=" + result);
			return result;
		} catch (IOException e) {
			logger.error("error", e);
			throw new CDGeneratorException(e);
		}
	}

	@Override
	public byte[] getCD(String choreographyName, String cdDir, String cdName) throws CDGeneratorException {
		
		try {
			String fileName = null;
			if (choreographyName!=null && !choreographyName.equals("")) {
				fileName = cdHomeRepository + File.separator + choreographyName + File.separator + File.separator + cdDir + File.separator + cdName;
			} else {
				fileName = cdHomeRepository + File.separator + cdDir + File.separator + cdName;
			}
			return FileUtils.readFileToByteArray(new File(fileName));
		} catch (IOException e) {
			logger.error("error", e);
			throw new CDGeneratorException(e);
		}
	}

}
