package it.univaq.di.choreos.synthesis.cdgenerator.impl.easyesb;

import it.univaq.di.choreos.synthesis.cdgenerator.utility.CDGeneratorUtility;
import it.univaq.di.choreos.synthesis.datamodel.CD;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.Map;

import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;

import org.apache.commons.io.FileUtils;

public class CDService {

	private static final String CONFIG_FILE = "config.xml";

	private static final String HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "\n" + "<Configuration xmlns=\"http://ebmwebsourcing.com/cli/schema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://ebmwebsourcing.com/cli/schema conf-schema.xsd\">" + "\n" + "  <Service url=\"http://localhost:8180/services/adminExternalEndpoint\">" + "\n" + "    <Action>" + "\n" + "       <Deploy>" + "\n";
	private static final String MIDDLE1 = "       </Deploy>" + "\n" + "    </Action>" + "\n";
	private static final String MIDDLE4 = "    <Action>" + "\n" + "       <Expose>" + "\n";
	private static final String SUFFIX = "       </Expose>" + "\n" + "    </Action>" + "\n" + "  </Service>" + "\n" + "</Configuration>";

	private CD cd;
	private String serviceEndpointName;
	private String serviceNamespace;
	private String serviceName;
	private String cdWsdl;

	public CDService(CD cd) {
		this.cd = cd;
		init();
	}

	private void init() {
		try {
			// get hold the WSDLFactory
			WSDLFactory factory = WSDLFactory.newInstance();
			// create an object to read the WSDL file
			WSDLReader reader = factory.newWSDLReader();

			// START
			// pass the URL to the reader for parsing and get back a WSDL
			// definiton
			String tempFolder = CDGeneratorUtility.getTempFolder();
			FileUtils.forceMkdir(new File(tempFolder));
			FileWriter file = new FileWriter(tempFolder + "tempWsdl.wsdl");
			file.write(cd.getWsdl());
			file.close();
			File fWsdl = new File(tempFolder + "tempWsdl.wsdl");
			String filePrefix = null;
			if (System.getProperty("os.name").toLowerCase().indexOf("win") >= 0) {
				filePrefix = "file:\\";
			} else {
				filePrefix = "file://";
			}
			Definition wsdlInstance = reader.readWSDL(null, filePrefix + fWsdl);
			FileUtils.deleteDirectory(new File(tempFolder));
			// END

			// set serviceNamespace
			serviceNamespace = wsdlInstance.getTargetNamespace();

			// set setServiceEndpointName and serviceName
			Map services = wsdlInstance.getServices();
			Iterator iterator = services.values().iterator();
			while (iterator.hasNext()) {
				Object element = iterator.next();
				if (element instanceof Service) {
					Service service = (Service) element;
					serviceName = service.getQName().getLocalPart();
					Iterator ports = service.getPorts().values().iterator();
					while (ports.hasNext()) {
						Port port = (Port) ports.next();
						serviceEndpointName = port.getName();
						// set cdEndpointName
						port.setName(getCdName());
					}
				}
			}
			StringWriter writer = new StringWriter();

			WSDLWriter wsdlWriter = factory.newWSDLWriter();
			wsdlWriter.writeWSDL(wsdlInstance, writer);

			cdWsdl = writer.toString();

		} catch (WSDLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private String getServiceEndpointName() {
		return serviceEndpointName;
	}

	private String getServiceNamespace() {
		return serviceNamespace;
	}

	private String getServiceName() {
		return serviceName;
	}

	public String getConfigContent() {
		StringBuilder builder = new StringBuilder();
		builder.append(HEADER);
		builder.append("           <MainResource>../../" + getCoordFileName() + "</MainResource>" + "\n");
		builder.append("           <Resource>../../" + getCDWSDLFileName() + "</Resource>" + "\n");
		builder.append("           <Resource>../../" + getPropertiesFileName() + "</Resource>" + "\n");
		builder.append(MIDDLE1);
		builder.append(MIDDLE4);
		builder.append("           <ServiceNamespace>" + getServiceNamespace() + "</ServiceNamespace>" + "\n");
		builder.append("           <ServiceName>" + getServiceName() + "</ServiceName>" + "\n");
		builder.append("           <EndpointName>" + getCdName() + "</EndpointName>" + "\n");
		builder.append(SUFFIX);
		return builder.toString();
	}

	public String getPropertiesContent() {
		StringBuilder builder = new StringBuilder();
		builder.append("endpoint=" + getServiceEndpointName());
		return builder.toString();

	}

	public String getCDWSDLContent() {
		return cdWsdl;
	}

	public String getCdName() {
		return cd.getCdName();
	}

	public String getPropertiesFileName() {
		return cd.getCdName() + ".properties";
	}

	public String getCDWSDLFileName() {
		return cd.getCdName() + ".wsdl";
	}
	
	public String getCoordFileName() {
		return cd.getCdName() + ".xmi";
	}
	
	public String getCoordContent() {
		return cd.getCoordModel();
	}

	public String getConfigFileName() {
		return CONFIG_FILE;
	}

}
