package it.univaq.di.choreos.synthesis.cdgenerator.impl.easyesb;

import it.univaq.di.choreos.synthesis.cdgenerator.CDGeneratorException;
import it.univaq.di.choreos.synthesis.cdgenerator.utility.CDGeneratorUtility;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorOutputStream;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EasyESBCDGenerator {

	final Logger logger = LoggerFactory.getLogger(EasyESBCDGenerator.class);

	public void generate(CDService cdService, String outputFileName) throws CDGeneratorException{
		logger.info("begin generating tar.gz= " + outputFileName);
		FileOutputStream fileOutputStream = null;
		BufferedOutputStream bufferedOutputStream = null;
		GzipCompressorOutputStream gzipCompressorOutputStream = null;
		TarArchiveOutputStream tarArchiveOutputStream = null;
		String tempFolder = CDGeneratorUtility.getTempFolder();

		try {
			/* create a TarArchiveOutputStream object. */
			fileOutputStream = new FileOutputStream(new File(outputFileName));
			bufferedOutputStream = new BufferedOutputStream(fileOutputStream);
			gzipCompressorOutputStream = new GzipCompressorOutputStream(bufferedOutputStream);
			tarArchiveOutputStream = new TarArchiveOutputStream(gzipCompressorOutputStream);
		
			FileUtils.forceMkdir(new File(tempFolder));

			/*
			 * Generate file config
			 */
			addFileToTargz(tarArchiveOutputStream, tempFolder, cdService.getConfigFileName(), cdService.getConfigContent());
				
			 // Generate file properties
			addFileToTargz(tarArchiveOutputStream, tempFolder, cdService.getPropertiesFileName(), cdService.getPropertiesContent());	
			
			// Generate file wsdl
			addFileToTargz(tarArchiveOutputStream, tempFolder, cdService.getCDWSDLFileName(), cdService.getCDWSDLContent());
			
			// Generate file coord
			addFileToTargz(tarArchiveOutputStream, tempFolder, cdService.getCoordFileName(), cdService.getCoordContent());
			
			logger.info("tar.gz file created successfully!!");
		} catch (FileNotFoundException e) {
			logger.error("FileNotFoundException", e);
			throw new CDGeneratorException(e);
		} catch (IOException e) {
			logger.error("IOException", e);
			throw new CDGeneratorException(e);
		}finally{
			try {
				tarArchiveOutputStream.close();
			} catch (Exception e) {}
			try {
				FileUtils.deleteDirectory(new File(tempFolder));
			} catch (IOException e) {}
			
		}
	}

	
	
	private void addFileToTargz(TarArchiveOutputStream tarArchiveOutputStream, String tempFolder, String resource, String content ) throws IOException{		
		FileWriter file = new FileWriter( tempFolder + resource );
		file.write(content);
		file.close();
		File f = new File(tempFolder + resource);

		logger.info("Adding File: " + f.getAbsolutePath());
		TarArchiveEntry archiveEntry = new TarArchiveEntry(f, resource);

		// Put the tar entry using putArchiveEntry.
		tarArchiveOutputStream.putArchiveEntry(archiveEntry);

		// Write the data to the tar file and close the input stream.
		byte data[] = null;
		data = content.getBytes();
		tarArchiveOutputStream.write(data, 0, data.length);
		f.delete();
		// close the archive entry.
		tarArchiveOutputStream.closeArchiveEntry();		
	}

}
