package it.univaq.di.choreos.synthesis.cdgenerator.rest;

import it.univaq.di.choreos.synthesis.cdgenerator.CDGenerator;
import it.univaq.di.choreos.synthesis.cdgenerator.CDGeneratorException;
import it.univaq.di.choreos.synthesis.cdgenerator.impl.CDGeneratorImpl;
import it.univaq.di.choreos.synthesis.configuration.Configuration;
import it.univaq.di.choreos.synthesis.datamodel.CD;

import java.net.URI;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Path("")
public class RestCDGenerator {

	private static Logger logger = LoggerFactory.getLogger(RestCDGenerator.class);

	private CDGenerator generator;

	public RestCDGenerator() {
		String cdHomeRepository = Configuration.get("CD_REPOSITORY");
		generator = new CDGeneratorImpl(cdHomeRepository);

	}

	@Path("/generate")
	@POST
	@Consumes(MediaType.APPLICATION_XML)
	public Response generate(CD cd, @Context UriInfo uriInfo) {

		try {
			String result = generator.generate(cd);
			UriBuilder uriBuilder = uriInfo.getBaseUriBuilder();
			
			uriBuilder = uriBuilder.path(RestCDGenerator.class).path("/getcd" + result);
			URI location = uriBuilder.build();
			return Response.created(location).build();

		} catch (CDGeneratorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}
	}

	
	@GET
	@Path("getcd/{choreographyName: .*}/{cdDir}/{cdName}")
	@Consumes("application/x-gzip")
	public Response getCD(@PathParam("choreographyName") String choreographyName, @PathParam("cdDir") String cdDir, @PathParam("cdName") String cdName, @Context UriInfo uriInfo) {
		logger.info("cdDir=" + cdDir);
		logger.info("cdName=" + cdName);

		try {
			byte[] cdContent = generator.getCD(choreographyName, cdDir, cdName);
			String header = "attachment; filename = " + cdName;
			return Response.ok(cdContent).header("content-disposition", header).build();

		} catch (CDGeneratorException e) {
			logger.error("error", e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}
	}

}
