package it.univaq.di.choreos.synthesis.servicediscovery.impl;

import it.univaq.di.choreos.synthesis.datamodel.AbstractParticipant;
import it.univaq.di.choreos.synthesis.datamodel.ELTSModel;
import it.univaq.di.choreos.synthesis.datamodel.Service;
import it.univaq.di.choreos.synthesis.datamodel.ServiceDiscoveryInput;
import it.univaq.di.choreos.synthesis.servicediscovery.ServiceDiscovery;
import it.univaq.di.choreos.synthesis.servicediscovery.ServiceDiscoveryException;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;


public class ServiceDiscoveryImpl implements ServiceDiscovery {
	private static final String[][] resourceWP6Service= 
		   {{"airport","airport", "airport"+File.separator+"Airport.wsdl", "airport"+File.separator+"Airport.elts", "http://sd-49168.dedibox.fr/DeployableServices/airport-service.jar"},
			{"airportbuscompany","airportbuscompany", "airportbuscompany"+File.separator+"AirportBusCompany.wsdl", "airportbuscompany"+File.separator+"AirportBusCompany.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportbuscompany-service.jar"},
			{"airportdisplayactuatorsaggregator","airportdisplayactuatorsaggregator", "airportdisplayactuatorsaggregator"+File.separator+"AirportDisplayActuatorsAggregator.wsdl", "airportdisplayactuatorsaggregator"+File.separator+"AirportDisplayActuatorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportdisplayactuatorsaggregator-service.jar"},
			{"airportinfraredsensorsaggregator","airportinfraredsensorsaggregator", "airportinfraredsensorsaggregator"+File.separator+"AirportInfraredSensorsAggregator.wsdl", "airportinfraredsensorsaggregator"+File.separator+"AirportInfraredSensorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportinfraredsensorsaggregator-service.jar"},
			{"airportnoisesensorsaggregator","airportnoisesensorsaggregator", "airportnoisesensorsaggregator"+File.separator+"AirportNoiseSensorsAggregator.wsdl", "airportnoisesensorsaggregator"+File.separator+"AirportNoiseSensorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportnoisesensorsaggregator-service.jar"},
			{"airportpressuresensorsaggregator","airportpressuresensorsaggregator", "airportpressuresensorsaggregator"+File.separator+"AirportPressureSensorsAggregator.wsdl", "airportpressuresensorsaggregator"+File.separator+"AirportPressureSensorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportpressuresensorsaggregator-service.jar"},
			{"airportsignactuatorsaggregator","airportsignactuatorsaggregator", "airportsignactuatorsaggregator"+File.separator+"AirportSignActuatorsAggregator.wsdl", "airportsignactuatorsaggregator"+File.separator+"AirportSignActuatorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportsignactuatorsaggregator-service.jar"},
			{"airportspeakeractuatorsaggregator","airportspeakeractuatorsaggregator", "airportspeakeractuatorsaggregator"+File.separator+"AirportSpeakerActuatorsAggregator.wsdl", "airportspeakeractuatorsaggregator"+File.separator+"AirportSpeakerActuatorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/airportspeakeractuatorsaggregator-service.jar"},
			{"airporttrafficcontrol","airporttrafficcontrol", "airtrafficcontrol"+File.separator+"AirTrafficControl.wsdl", "airtrafficcontrol"+File.separator+"AirTrafficControl.elts", "airtrafficcontrolServiceURI"},
			{"bookableamenity","bookableamenity", "bookableamenity"+File.separator+"BookableAmenity.wsdl", "bookableamenity"+File.separator+"BookableAmenity.elts", "http://sd-49168.dedibox.fr/DeployableServices/bookableamenity-service.jar"},
			{"luggagehandlingcompany","luggagehandlingcompany", "luggagehandlingcompany"+File.separator+"LuggageHandlingCompany.wsdl", "luggagehandlingcompany"+File.separator+"LuggageHandlingCompany.elts", "http://sd-49168.dedibox.fr/DeployableServices/luggagehandlingcompany-service.jar"},
			{"middisplayactuatorsaggregator","middisplayactuatorsaggregator", "middisplayactuatorsaggregator"+File.separator+"MIDDisplayActuatorsAggregator.wsdl", "middisplayactuatorsaggregator"+File.separator+"MIDDisplayActuatorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/middisplayactuatorsaggregator-service.jar"},
			{"midlocationsensorsaggregator","midlocationsensorsaggregator", "midlocationsensorsaggregator"+File.separator+"MIDLocationSensorsAggregator.wsdl", "midlocationsensorsaggregator"+File.separator+"MIDLocationSensorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/midlocationsensorsaggregator-service.jar"},
			{"midmicrophonesensorsaggregator","midmicrophonesensorsaggregator", "midmicrophonesensorsaggregator"+File.separator+"MIDMicrophoneSensorsAggregator.wsdl", "midmicrophonesensorsaggregator"+File.separator+"MIDMicrophoneSensorsAggregator.elts", "http://sd-49168.dedibox.fr/DeployableServices/midmicrophonesensorsaggregator-service.jar"},
			{"securitycompany","securitycompany", "securitycompany"+File.separator+"SecurityCompany.wsdl", "securitycompany"+File.separator+"SecurityCompany.elts", "http://sd-49168.dedibox.fr/DeployableServices/securitycompany-service.jar"},
			{"standandgatemanagement","standandgatemanagement", "standandgatemanagement"+File.separator+"StandAndGateManagement.wsdl", "standandgatemanagement"+File.separator+"StandAndGateManagement.elts", "http://sd-49168.dedibox.fr/DeployableServices/standandgatemanagement-service.jar"}
			};
	
	private static final String[][] resourceWP7Service= 
		   {{"BasicCommunicationDevice", "basiccommunicationdevice", "basiccommunicationdevice"+File.separator+"basiccommunicationdevice.wsdl", "basiccommunicationdevice"+File.separator+"basiccommunicationdevice.elts", "basiccommunicationdeviceServiceURI"},
			{"Client", "client", "client"+File.separator+"client.wsdl", "client"+File.separator+"client.elts", "clientServiceURI"},
			{"InStoreTotem", "instoretotem", "instoretotem"+File.separator+"instoretotem.wsdl", "instoretotem"+File.separator+"instoretotem.elts", "instoretotemServiceURI"},
			{"MarketingApplication", "marketingapplication", "marketingapplication"+File.separator+"marketingapplication.wsdl", "marketingapplication"+File.separator+"marketingapplication.elts", "marketingapplicationServiceURI"},
			{"MarketingManager", "marketingmanager", "marketingmanager"+File.separator+"marketingmanager.wsdl", "marketingmanager"+File.separator+"marketingmanager.elts", "marketingmanagerServiceURI"},
			{"SelfCheckOutMachine", "selfcheckoutmachine", "selfcheckoutmachine"+File.separator+"selfcheckoutmachine.wsdl", "selfcheckoutmachine"+File.separator+"selfcheckoutmachine.elts", "selfcheckoutmachineServiceURI"},
			{"ShopEntrance", "shopentrance", "shopentrance"+File.separator+"shopentrance.wsdl", "shopentrance"+File.separator+"shopentrance.elts", "ServiceURI"},
			{"ShoppingAssistants", "shoppingassistants", "shoppingassistants"+File.separator+"shoppingassistantapp.wsdl", "shoppingassistants"+File.separator+"shoppingassistants.elts", "shoppingassistantsServiceURI"},
			{"SmartCart", "smartcart", "smartcart"+File.separator+"smartcart.wsdl", "smartcart"+File.separator+"smartcart.elts", "smartcartServiceURI"}
			};
	
	private static final String[][] resourceWP8Service= 
		   {{"AIM Adaptive ItineraryManager","itinerarymanager", "itinerarymanager"+File.separator+"itinerarymanager.wsdl", "itinerarymanager"+File.separator+"itinerarymanager.elts", "itinerarymanagerURI"},
			{"Notification Manager","notificationmanager", "notificationmanager"+File.separator+"notificationmanager.wsdl", "notificationmanager"+File.separator+"notificationmanager.elts", "notificationmanagerURI"},
			{"User Manager Service", "usermanager","usermanager"+File.separator+"usermanager.wsdl", "usermanager"+File.separator+"usermanager.elts", "usermanagerURI"},
			{"Things Registry","thingsregistry", "thingsregistry"+File.separator+"thingsregistry.wsdl", "thingsregistry"+File.separator+"thingsregistry.elts", "thingsregistryURI"},
			{"Social Proximity Service","socialproximity", "socialproximity"+File.separator+"socialproximity.wsdl", "socialproximity"+File.separator+"socialproximity.elts", "socialproximityURI"},
			{"Citizen MID","thingslocation", "thingslocation"+File.separator+"thingslocation.wsdl", "thingslocation"+File.separator+"thingslocation.elts", "thingslocationURI"},
			};
	
	private Map<String, List<Service>> servicesWP6Abstract;
	private Map<String, List<Service>> servicesWP7Abstract;
	private Map<String, List<Service>> servicesWP8Abstract;
	private Map<String, List<Service>> servicesWP6Concrete;
	private Map<String, List<Service>> servicesWP7Concrete;
	private Map<String, List<Service>> servicesWP8Concrete;
	
	private String resourcesWP6 = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator + "wp6"+ File.separator;
	private String resourcesWP7 = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator + "wp7"+ File.separator;
	private String resourcesWP8 = "." + File.separator + "src" + File.separator + "main" + File.separator + "resources" + File.separator + "wp8"+ File.separator;

	public ServiceDiscoveryImpl() {
		servicesWP6Abstract = new HashMap<String, List<Service>>();
		for (int i =0; i<resourceWP6Service.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			try {
				service.setWsdl(FileUtils.readFileToString(new File(resourcesWP6+resourceWP6Service[i][2])));
				ELTSModel eltsModel = new ELTSModel(FileUtils.readFileToString(new File(resourcesWP6+resourceWP6Service[i][3])), resourceWP6Service[i][1]);
				AbstractParticipant abstractParticipant = new AbstractParticipant();
				abstractParticipant.setAbstractName(resourceWP6Service[i][0]);
				abstractParticipant.setConcreteName(resourceWP6Service[i][1]);
				service.setELTS(eltsModel);
				service.setServiceURI(resourceWP6Service[i][4]);
				service.setLegacy(false);
				servicesDiscovery.add(service);
				servicesWP6Abstract.put(resourceWP6Service[i][0], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}servicesWP6Abstract = new HashMap<String, List<Service>>();
		}
		
		servicesWP6Concrete = new HashMap<String, List<Service>>();
		for (int i =0; i<resourceWP6Service.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			try {
				service.setWsdl(FileUtils.readFileToString(new File(resourcesWP6+resourceWP6Service[i][2])));
				ELTSModel eltsModel = new ELTSModel(FileUtils.readFileToString(new File(resourcesWP6+resourceWP6Service[i][3])), resourceWP6Service[i][1]);
				AbstractParticipant abstractParticipant = new AbstractParticipant();
				abstractParticipant.setAbstractName(resourceWP6Service[i][0]);
				abstractParticipant.setConcreteName(resourceWP6Service[i][1]);
				service.setELTS(eltsModel);
				service.setServiceURI(resourceWP6Service[i][4]);
				service.setLegacy(false);
				servicesDiscovery.add(service);
				servicesWP6Concrete.put(resourceWP6Service[i][1], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		
		servicesWP7Abstract = new HashMap<String, List<Service>>();
		for (int i =0; i<resourceWP7Service.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			try {
				service.setWsdl(FileUtils.readFileToString(new File(resourcesWP7+resourceWP7Service[i][2])));
				ELTSModel eltsModel = new ELTSModel(FileUtils.readFileToString(new File(resourcesWP7+resourceWP7Service[i][3])), resourceWP7Service[i][1]);
				AbstractParticipant abstractParticipant = new AbstractParticipant();
				abstractParticipant.setAbstractName(resourceWP7Service[i][0]);
				abstractParticipant.setConcreteName(resourceWP7Service[i][1]);
				service.setAbstractParticipant(abstractParticipant);
				service.setELTS(eltsModel);
				service.setServiceURI(resourceWP7Service[i][4]);
				servicesDiscovery.add(service);
				service.setLegacy(true);
				servicesWP7Abstract.put(resourceWP7Service[i][0], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		servicesWP7Concrete= new HashMap<String, List<Service>>();
		for (int i =0; i<resourceWP7Service.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			try {
				service.setWsdl(FileUtils.readFileToString(new File(resourcesWP7+resourceWP7Service[i][2])));
				ELTSModel eltsModel = new ELTSModel(FileUtils.readFileToString(new File(resourcesWP7+resourceWP7Service[i][3])), resourceWP7Service[i][1]);
				AbstractParticipant abstractParticipant = new AbstractParticipant();
				abstractParticipant.setAbstractName(resourceWP7Service[i][0]);
				abstractParticipant.setConcreteName(resourceWP7Service[i][1]);
				service.setAbstractParticipant(abstractParticipant);
				service.setELTS(eltsModel);
				service.setServiceURI(resourceWP7Service[i][4]);
				servicesDiscovery.add(service);
				service.setLegacy(true);
				servicesWP7Concrete.put(resourceWP7Service[i][1], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		servicesWP8Abstract = new HashMap<String, List<Service>>();
		for (int i =0; i<resourceWP8Service.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			try {
				service.setWsdl(FileUtils.readFileToString(new File(resourcesWP8+resourceWP8Service[i][2])));
				ELTSModel eltsModel = new ELTSModel(FileUtils.readFileToString(new File(resourcesWP8+resourceWP8Service[i][3])), resourceWP8Service[i][1]);
				AbstractParticipant abstractParticipant = new AbstractParticipant();
				abstractParticipant.setAbstractName(resourceWP8Service[i][0]);
				abstractParticipant.setConcreteName(resourceWP8Service[i][1]);
				service.setAbstractParticipant(abstractParticipant);
				service.setELTS(eltsModel);
				service.setServiceURI(resourceWP8Service[i][4]);
				servicesDiscovery.add(service);
				service.setLegacy(true);
				servicesWP8Abstract.put(resourceWP8Service[i][0], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		servicesWP8Concrete = new HashMap<String, List<Service>>();
		for (int i =0; i<resourceWP8Service.length; i++){
			List<Service> servicesDiscovery = new ArrayList<Service>();
			Service service = new Service();
			try {
				service.setWsdl(FileUtils.readFileToString(new File(resourcesWP8+resourceWP8Service[i][2])));
				ELTSModel eltsModel = new ELTSModel(FileUtils.readFileToString(new File(resourcesWP8+resourceWP8Service[i][3])), resourceWP8Service[i][1]);
				AbstractParticipant abstractParticipant = new AbstractParticipant();
				abstractParticipant.setAbstractName(resourceWP8Service[i][0]);
				abstractParticipant.setConcreteName(resourceWP8Service[i][1]);
				service.setAbstractParticipant(abstractParticipant);
				service.setELTS(eltsModel);
				service.setServiceURI(resourceWP8Service[i][4]);
				servicesDiscovery.add(service);
				service.setLegacy(true);
				servicesWP8Concrete.put(resourceWP8Service[i][1], servicesDiscovery);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public List<Service> discoverServices(ServiceDiscoveryInput serviceDiscoveryInput) throws ServiceDiscoveryException {
		if (serviceDiscoveryInput.getChoreographyName().toLowerCase().contains("wp6")){
			if (servicesWP6Abstract.get(serviceDiscoveryInput.getParticipant()) != null){
				return servicesWP6Abstract.get(serviceDiscoveryInput.getParticipant());
			}else{
				return servicesWP6Concrete.get(serviceDiscoveryInput.getParticipant());
			}
		}else if (serviceDiscoveryInput.getChoreographyName().toLowerCase().contains("wp7")){
			if (servicesWP7Abstract.get(serviceDiscoveryInput.getParticipant()) != null){
				return servicesWP7Abstract.get(serviceDiscoveryInput.getParticipant());
			}else{
				return servicesWP7Concrete.get(serviceDiscoveryInput.getParticipant());
			}
		}else if (serviceDiscoveryInput.getChoreographyName().toLowerCase().contains("wp8")){
			if (servicesWP8Abstract.get(serviceDiscoveryInput.getParticipant()) != null){
				return servicesWP8Abstract.get(serviceDiscoveryInput.getParticipant());
			}else{
				return servicesWP8Concrete.get(serviceDiscoveryInput.getParticipant());
			}
		}
		
		return new ArrayList<Service>();
	}

}
